<?php

/**
 * crawler - fake content for crawlers
 *
 * Purpose: Feed crawlers with content from other websites
 *          f.e. upload a sitemap to google with the link http://your_roundcube/?_action=plugin.crawler
 *          Tools: http://www.webconfs.com/search-engine-spider-simulator.php
 *                 http://www.seobench.com/search-engine-crawler-simulator/index.php
 *
 * @version 1.5.2 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 **/ 

class crawler extends rcube_plugin
{
  public $task = 'login|logout';
  public $noframe = true;
  public $noajax = true;
  
  private $url;
  private $scheme;
  private $host;
  private $path;
  private $content;
  
  /* unified plugin properties */
  static private $plugin = 'crawler';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.5.2';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
 
  function init()
  {
    $this->add_hook('startup', array($this, 'check_crawler'));
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function check_crawler()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }    
    if($_GET['_action'] == 'plugin.crawler'){
      // by request
      $this->crawler_detected();
    }
    else if(preg_match("/yahoo/i", $_SERVER['HTTP_USER_AGENT'])){
      // assume crawler
      $this->crawler_detected();
    }
    else if(preg_match("/yandexbot/i", $_SERVER['HTTP_USER_AGENT'])){
      // assume crawler
      $this->crawler_detected();
    }
    else if(preg_match("/googlebot/i", $_SERVER['HTTP_USER_AGENT'])){
      // assume crawler
      $this->crawler_detected();
    }
    else if(preg_match("/mozilla/i", $_SERVER['HTTP_USER_AGENT'])){
      // common browsers
      return;
    }
    else if(preg_match("/opera/i", $_SERVER['HTTP_USER_AGENT'])){
      // common browsers
      return;
    }
    else{
      // assume crawler
      $this->crawler_detected();
    }
  }

  function crawler_detected()
  {
    $rcmail = rcmail::get_instance();
    $urls = $rcmail->config->get('crawl_urls',array());
    if(count($urls) > 0){
      srand(microtime()*1000000);
      $url = $urls[rand(0,count($urls)-1)];
      write_log('crawler', $url . "\t" . $_SERVER['REMOTE_ADDR'] . "\t". $_SERVER['HTTP_USER_AGENT'] . "\t" . $_SERVER['REMOTE_HOST']);
      if($_SERVER['QUERY_STRING'] != ""){
        header("HTTP/1.0 404 Not Found"); 
        exit;
      }
      $this->show_content($url);
    }
  }
  
  function show_template()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->send("crawler.crawler");
  }  

  function show_content($url)
  {
    $arr = parse_url($url);
    $this->scheme = $arr['scheme'] . "://";
    $this->host = $arr['host'];
    $this->path = $arr['path'];
    $this->url = slashify($url);
    $ch = curl_init($url);
    curl_setopt($ch,CURLOPT_HEADER,0);
    curl_setopt($ch,CURLOPT_RETURNTRANSFER,1);
    $cr = curl_exec($ch);

    $this->content = $cr;

    preg_replace_callback("/HREF=\"[^(http)]/i", array( &$this, 'preg_callback_href_dq'), $this->content); 
    preg_replace_callback("/HREF='[^(http)]/i", array( &$this, 'preg_callback_href_sq'), $this->content); 
    preg_replace_callback("/SRC=\"[^(http)]/i", array( &$this, 'preg_callback_src_dq'), $this->content); 
    preg_replace_callback("/SRC='[^(http)]/i", array( &$this, 'preg_callback_src_sq'), $this->content);
    preg_replace_callback("/URL\(\"[^(http)]/i", array( &$this, 'preg_callback_url_sq'), $this->content); 
    preg_replace_callback("/URL\('[^(http)]/i", array( &$this, 'preg_callback_url_sq'), $this->content);

    echo $this->content;
    exit;
  }

  function preg_callback_href_dq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"href=\"" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"href=\"" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }
 
  function preg_callback_href_sq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"href='" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"href='" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }
  
  function preg_callback_src_dq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"src=\"" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"src=\"" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }

  function preg_callback_src_sq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"src='" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"src='" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }

  function preg_callback_url_dq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"url(\"" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"url(\"" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }

  function preg_callback_url_sq($matches) 
  {
    if(substr($matches[0],strlen($matches[0])-1) != "/"){
      $this->content = str_replace($matches[0],"url('" . $this->url . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    else{
      $this->content = str_replace($matches[0],"url('" . $this->scheme . $this->host . substr($matches[0], strlen($matches[0])-1),$this->content);
    }
    return;
  }

}
?>