<?php

/**
 * dblog
 *
 * Log to database
 *
 * @version 1.8 - 06.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/

/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * IMPORTANT: This plugin can't use global_config!
 *            Configure this plugin separately in plugin folder.
 *
 **/ 

class dblog extends rcube_plugin
{
  /* unified plugin properties */
  static private $plugin = 'dblog';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = "IMPORTANT: Configure this plugin separately. Don't use <i>global_config</i>.";
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.8';
  static private $date = '06-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
  );
  static private $incompat = null;
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $this->add_hook('write_log', array($this, 'write_log'));
    $this->add_hook('startup', array($this, 'startup'));
    $this->load_config();
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin::about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
      'incompat' => self::$incompat,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  function startup($args){

    if(!empty($_SERVER['QUERY_STRING'])){
      $query = $_SERVER['QUERY_STRING'];
    }
    else{
      $query = 'RoundCube Root';
    }
    $this->write_log(array('name' => 'access', 'line' => $_SERVER['REQUEST_METHOD'] . ': ' . $query));
    return $args;
  }

  function write_log($args){

    $rcmail = rcmail::get_instance();

    // check if we have to log to database
    $logs = array_flip($rcmail->config->get('logs', array('access', 'errors', 'userlogins', 'sendmail')));
    
    if(!isset($logs[$args['name']]))
      return $args;

    if(isset($_SESSION['user_id'])){
      $user_id = $_SESSION['user_id'];
    }
    else {
      $user_id = 0;  
    }

    if(isset($_SESSION['username'])){
      $username = $_SESSION['username'];
    }
    else {
      $username = 'unauthenticated';
    }

    if($rcmail->config->get('log_ips', false)){
      $ip = $this->getVisitorIP();
    }
    else{
      $ip = '0.0.0.0';
    }

    $line = $args['line'];

    $query = "INSERT INTO " . get_table_name('logs') . "(log, userip, hostip, line, user_id, username) VALUES (?, ?, ?, ?, ?, ?);";

    $ret = $rcmail->db->query($query,
      $args['name'],
      $ip,
      $_SERVER['SERVER_ADDR'],
      $line,
      $user_id,
      $username);
    
    // do not log to file  
    if($ret)
      $args['abort'] = true;
    
    $max = $rcmail->config->get('logs_export');  
    if(is_numeric($max) && empty($_SESSION['logs_db_rows'])){  
      $query = "SELECT COUNT(id) FROM " . get_table_name('logs') . ";";
      $ret = $rcmail->db->query($query);
      $ret = $rcmail->db->fetch_assoc($ret);
      $count = $ret['count(id)'];
      // set session flag
      $_SESSION['logs_db_rows'] = $count;
      if($count > $max){
        $dir = $rcmail->config->get('log_dir');
        $query = "SELECT * FROM " . get_table_name('logs') . ";";
        $ret = $rcmail->db->query($query);
        $sql_arr = array();
        while($res = $rcmail->db->fetch_assoc($ret))
          $sql_arr[] = $res;
        $content = "";
        foreach($sql_arr as $key => $entry){
          foreach($entry as $line => $val){
            $content .= $val . " ";
          }
          $content .= "\r\n";
        }
        $file = $dir . "/" . "dblogs." . time() . ".log";
        if(@file_put_contents($file,$content)){
          // truncate table
          $query = "TRUNCATE TABLE " . get_table_name('logs') . ";";
          $rcmail->db->query($query);
        }
      }
    }

    return $args;

  }

  function getVisitorIP() { 

    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 

  }

}

?>