<?php

/**
 * Demo plugin
 *
 *
 * @version 2.7.2 - 07.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * NOTICE: "./plugins/demologin/default_messages/" must have read access !!!
 *
 * Recommended Plugins: (*) taskbar
 *
 **/
 
class demologin extends rcube_plugin
{
  public $task = 'login|logout|mail|settings';
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'demologin';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Important Update Notes for Versions 2.7 and newer: <a href="http://mirror.mail4us.net/docs/demologin.html" target="_new">Click here</a>';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.7.2';
  static private $date = '07-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1',
    'recommended_plugins' => array('taskbar' => 'config'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_texts('localization/');
    $this->add_hook('startup', array($this, 'startup'));
    $this->add_hook('login_after', array($this, 'login_after'));
    $this->add_hook('authenticate', array($this, 'authenticate'));
    $this->add_hook('session_destroy', array($this, 'session_destroy'));
    $this->add_hook('message_outgoing_headers', array($this, 'redirect_to_myself'));
    $this->add_hook('render_page', array($this, 'render_page'));
    /* garbage collection */
    $rand = rand(ini_get('session.gc_probability') ,ini_get('session.gc_divisor'));
    $match = round((ini_get('session.gc_divisor') - ini_get('session.gc_probability') + 1) /  2, 0);
    if($rand == $match){
      $this->gc();
    }
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function render_page($p){
    $rcmail = rcmail::get_instance();
    if($_SESSION['demouser']){
      if($rcmail->config->get('demo_target_group', 'admins') != 'admins'){
        $_SESSION['plugin.newuserdialog'] = false;
      }
      if($p['template'] == 'plugin'){
        $this->include_script('demologin.js');
        $rcmail->output->add_label('demologin.passworddisabled');
      }
    }
    if(strtolower($this->get_demo($_SESSION['username'])) == strtolower(sprintf($rcmail->config->get('demo_user_account'),""))){
      $query = 'UPDATE ' . get_table_name('demo') . ' SET changed=? WHERE sess_id=?';
      $rcmail->db->query($query, date('Y-m-d H:i:s', time()), session_id());
    }
    return $p;
  }
  
  function session_destroy($a)
  {
    $rcmail = rcmail::get_instance();
    $query = "DELETE FROM " . get_table_name('demo') . " WHERE sess_id=?";
    $ret = $rcmail->db->query($query, session_id());
    return $a;
  }
  
  function startup($args)
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->set_env('demo_user', sprintf($rcmail->config->get('demo_user_account'),""));
    if($_SESSION['demouser']){
      // change action to login
      if ($args['task'] == 'mail' && empty($args['action']) && empty($_SESSION['user_id']) && !empty($_POST['_user'])){
        $args['action'] = 'login';
      }
      if ($args['task'] == 'logout'){
        $this->reset_account(true);
      }
    }

    return $args;
  }
  
  function login_after($args)
  {
    if($_SESSION['demouser']){
      $rcmail = rcmail::get_instance();
      $query = 'INSERT INTO ' . get_table_name('demo') . ' (sess_id, created, changed, ip, vars) VALUES (?, ?, ?, ?, ?)';
      $rcmail->db->query($query, session_id(), date('Y-m-d H:i:s', time()), date('Y-m-d H:i:s', time()), $this->getVisitorIP(), $_SESSION['username']);
      if($rcmail->config->get('demo_target_group', 'admins') != 'admins'){
        $this->reset_account();
      }
      if(class_exists('newuser')){
        newuser::newuser_send_welcome(array());
      }
    }
    return $args;
  }

  function authenticate($args)
  {
    $rcmail = rcmail::get_instance();
    
    if($args['user'] == sprintf($rcmail->config->get('demo_user_account'),"")){
      $occupied = $this->list_occupied_accounts();
      $accounts = array();
      if(is_array($occupied))
        $accounts = array_flip($occupied);
      $demo_accounts = array();
      $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),"");
      if($rcmail->config->get('demo_accounts') > 0){
        for($i=0;$i<=$rcmail->config->get('demo_accounts');$i++){
          $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),$i);
        }
        $user = false;
        foreach($demo_accounts as $key => $val){
          if(!isset($accounts[$val])){
            // ok, no user is active on this demo
            $user = $val;
            break;
          }
        }

        if(!$user){
          $rcmail->output->command('display_message',rcube_label('demoaccountsoccupied', 'demologin'),'error');
          $rcmail->output->send('login');
          exit;
        }
        else{
          $args['user'] = $user;
        }
      }
      else{
        $args['user'] = sprintf($rcmail->config->get('demo_user_account'),"");
      }
      $args['pass'] = $rcmail->config->get('demo_user_password');
      $args['host'] = $rcmail->config->get('demo_user_host');
      $_SESSION['demouser'] = $args['user'];
      $args['valid'] = true;
      $args['cookiecheck'] = false;
    }
 
    return $args;
  }
  
  function list_occupied_accounts()
  {  
    $rcmail = rcmail::get_instance();
    
    $sql_result = $rcmail->db->query(
      "SELECT * FROM ".get_table_name('demo').";"
    );
    
    $temparr = explode("%s",$rcmail->config->get('demo_user_account'));
    $demo_account = $temparr[0];
    
    $result = array();
    $timeouts = array();
    while ($sql_arr = $rcmail->db->fetch_assoc($sql_result)) {
      $result[] = $sql_arr['vars'];
      if(strtotime($sql_arr['changed']) + $rcmail->config->get('demo_session_timeout', 180) < time()){
        $timeouts[] = $sql_arr['sess_id'];
      }
    }
    
    foreach($timeouts as $key => $val){
      $sql_result = $rcmail->db->query(
        "DELETE FROM ".get_table_name('demo')." WHERE sess_id='" . $val . "'"
      );
    }

    return $result;
  }
  
  function reset_account($purge = false)
  {
    $rcmail = rcmail::get_instance();

    $user = $rcmail->user->data['username'];
    
    // should not be necessary ... but make double sure not to purge
    //                             a prodctive account;
    $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),"");
    for($i=0;$i<=$rcmail->config->get('demo_accounts');$i++){
      $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),$i);
    }
    $demo_accounts = array_flip($demo_accounts);
    if(!isset($demo_accounts[$user])){
      return;
    }

    // purge default folders
    $rcmail->imap_connect();
    $default_mailboxes = $rcmail->config->get('default_imap_folders');
    if(!is_array($default_mailboxes)){
      $default_mailboxes = $rcmail->config->get('default_folders', array('INBOX'));
    }
    foreach($default_mailboxes as $key => $val){
      // SELECT will set messages count for clearFolder()
      if($rcmail->imap->conn->select($val)) {
        $rcmail->imap->conn->clearFolder($val);
      }
    }

    // reset folder list
    $mailboxes = array_flip($rcmail->imap->list_mailboxes());
    $default_mailboxes = array_flip($default_mailboxes);
    foreach($mailboxes as $key => $val){
      if(!isset($default_mailboxes[$key])){
        $rcmail->imap->delete_mailbox($key);
      }
    }

    // import default messages
    $msgs = array();
    $msgs = scandir("./plugins/demologin/default_messages");
    foreach($msgs as $key => $val){
      // skip pointers
      if($val != "." && $val != ".."){
        if($msg = @file_get_contents("./plugins/demologin/default_messages/" . $val)){
          $rcmail->imap->save_message('INBOX', rtrim($msg));
        }
      }
    }
    
    if($purge){
      $this->delete_user();
    }
  }
  
  function delete_user()
  {
    $rcmail = rcmail::get_instance();
    // reset settings by deleting roundcube user
    $sql_result = $rcmail->db->query(
      "DELETE FROM ".get_table_name('users')." WHERE user_id=?",
      $rcmail->user->ID
    );
  }
  
  function redirect_to_myself($a)
  {
  
    $rcmail = rcmail::get_instance();
    $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),"");
    for($i=0;$i<=$rcmail->config->get('demo_accounts');$i++){
       $demo_accounts[] = sprintf($rcmail->config->get('demo_user_account'),$i);
    }
    $demo_accounts = array_flip($demo_accounts);
    if(isset($demo_accounts[$rcmail->user->data['username']])){
      $this->add_texts('localization/');
      $rcmail->output->command('display_message',rcube_label('sendingdisabled','demologin'),'error');
      $rcmail->output->send('iframe');
    }
    
    return $a;  
  }
  
  private function get_demo($string)
  {
    $temparr = explode("@",$string);  
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];
  }
  
  private function getVisitorIP()
  { 
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 
    return $visitorIP; 
  }
  
  private function gc()
  {
    $rcmail = rcmail::get_instance();
    // reset settings by deleting roundcube user
    $sql_result = $rcmail->db->query(
      "DELETE FROM ".get_table_name('demo')." WHERE created<?",
      date('Y-m-d H:i:s', time() - 86400)
    );
  }
}

?>