<?php
/**
 * detach_attachments
 *
 * @version 2.3 - 17.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 * 
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/
 
/**
 *
 * Note: #1- dl folder must have read/write access
 *       #2- .htaccess in Roundcube folder denies file listing
 *           If you don't use the default .htaccess make sure
 *           to prevent file listing of dl folder
 *
 **/
 
class detach_attachments extends rcube_plugin
{
  public $task = 'login|mail|settings';
  private $dir;
  private $lifetime;
  
  /* unified plugin properties */
  static private $plugin = 'detach_attachments';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Important Update Notes for Versions 2.x: <a href="http://mirror.mail4us.net/docs/detach_attachments.html" target="_new">Click here</a>';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.3';
  static private $date = '17-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1',
    'required_plugins' => array(
      'database_attachments' =>  'require_plugin',
    ),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init(){
    // don't init for cron jobs
    if(!isset($_SERVER['REMOTE_ADDR'])){
      return;
    }
    $rcmail = rcmail::get_instance();
    $this->require_plugin('database_attachments');
    $this->dir = slashify($rcmail->config->get('dl_dir', 'plugins/detach_attachments/dl'));
    $this->lifetime = $rcmail->config->get('dl_expires', 30);
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_hook('attachment_upload', array($this, 'detach'));
    $this->add_hook('message_compose', array($this, 'reset'));
    $this->add_hook('startup', array($this, 'download'));
    if($rcmail->task == 'mail' && $rcmail->action == 'get'){
      $this->add_hook('render_page', array($this, 'render_page'));
    }
    
    $this->register_action('plugin.dta_import', array($this, 'import'));
    $this->register_action('plugin.dta_counts', array($this, 'counts'));
    $this->register_action('plugin.dla_delete', array($this, 'delete'));
    
    $dont_override = $rcmail->config->get('dont_override', array());
    if (!in_array('cum_upload_detach', $dont_override)) {
      $this->add_hook('preferences_list', array($this, 'prefs_table'));
      $this->add_hook('preferences_save', array($this, 'save_prefs'));
    }
    else{
       $rcmail->config->set('cum_upload_detach', $rcmail->config->get('cum_upload_default', 5));
    }

    if($rcmail->action == 'remove-attachment'){
      $id = get_input_value('_id', RCUBE_INPUT_GPC);
      $file = str_replace('rcmfile', '', get_input_value('_file', RCUBE_INPUT_GPC));
      $sql_result = $rcmail->db->query(
        "SELECT data
         FROM ".get_table_name('cache')."
         WHERE  user_id=?
         AND    cache_key=?",
        $_SESSION['user_id'],
        $file
      );
      if($sql_arr = $rcmail->db->fetch_assoc($sql_result)){
        $size = strlen(base64_decode($sql_arr['data']));
        $_SESSION['attachments_total'][$id] = $_SESSION['attachments_total'][$id] - $size;
      }
      else{
        $sql_result = $rcmail->db->query(
          "SELECT data
           FROM ".get_table_name('attachments')."
           WHERE  user_id=?
           AND    cache_key LIKE '" . $file . "%'",
          $_SESSION['user_id']
        );
        if($sql_arr = $rcmail->db->fetch_assoc($sql_result)){
          $size = strlen(base64_decode($sql_arr['data']));
          $_SESSION['attachments_total'][$id] = $_SESSION['attachments_total'][$id] - $size;
        }
      }
    }
    
    /* garbage collection */
    $rand = rand(ini_get('session.gc_probability') ,ini_get('session.gc_divisor'));
    $match = round((ini_get('session.gc_divisor') - ini_get('session.gc_probability') + 1) /  2, 0);
    if($rand == $match){
      $this->gc($this->lifetime);
    }
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function prefs_table($args){
    if($args['section'] == 'compose'){
      $this->add_texts('localization', false);
      $rcmail = rcmail::get_instance();
      $field_id = 'rcmfd_detach_attachments';
      $select = new html_select(array('name' => '_cum_upload_detach', 'id' => $field_id));
      $select->add($this->gettext('never'), -1);
      $select->add($this->gettext('always'), 0);
      $multiplier = $rcmail->config->get('cum_upload_limit_resolution', 1);
      for($i = $multiplier; $i <= $rcmail->config->get('cum_upload_detach_max_limit', $multiplier * 5); $i += $multiplier){
        $select->add($this->gettext('detach_attachments.cumsize') . ' > ' . $i . ' MB', $i);
      }
      $args['blocks']['main']['options']['detachattachments']['title'] = Q($this->gettext('storeattachments'));
      $args['blocks']['main']['options']['detachattachments']['content'] = $select->show((int)$rcmail->config->get('cum_upload_detach', 5), array('name' => "_cum_upload_detach"));
    }
    return $args;
  }
  
  function save_prefs($args)  {
    if($args['section'] == 'compose'){
      $rcmail = rcmail::get_instance();
      $size = get_input_value('_cum_upload_detach', RCUBE_INPUT_POST);
      $args['prefs']['cum_upload_detach'] = $size;
      return $args;
    }
  }
  
  function gc($lifetime){
    $rcmail = rcmail::get_instance();
    $lifetime = (int)$lifetime + 5;
    $sql_result = $rcmail->db->query(
      "SELECT cache_id, created
       FROM ".get_table_name('attachments')
    );
    $result = array();
    while($sql_arr = $rcmail->db->fetch_assoc($sql_result)) {
      if(time() - strtotime($sql_arr['created']) > $lifetime * 86400){
        $result[] = $sql_arr;
      }
    }
    foreach($result as $prop){
      $rcmail->db->query(
        "DELETE FROM ".get_table_name('attachments')."
         WHERE  cache_id=?",
        $prop['cache_id']
      );
    }
    $temp_dir = slashify(ini_get('upload_tmp_dir'));
    if(!$temp_dir){
      $temp_dir = slashify($rcmail->config->get('temp_dir','temp'));
    }
    $files = scandir($temp_dir);
    foreach($files as $file){
      $temp = explode('.', $file);
      if(strtolower($temp[count($temp) - 1]) == '.dta' || strtolower($temp[count($temp) - 1]) == 'tmp'){ 
        if(time() - filemtime($temp_dir . $file) > 86400){
          @unlink($temp_dir . $file);
        }
      }
    }
  }
  
  function render_page($p){
    if($p['template'] == 'messagepart'){
      $this->include_script('detach_attachments.js');
      $this->add_texts('localization/', true);
      rcmail::get_instance()->output->set_env('user_hash', md5($_SESSION['user_id']));
    }
  }
  
  function import(){
    $rcmail = rcmail::get_instance();
    $dir = $this->dir;
    if(is_dir($dir)){
      if($dh = opendir($dir)){
        @unlink($dir . 'dl.txt');
        echo "<html><head><title>detach_attachments></title></head><body><h3>Plugin detach_attachments Migration</h3><hr />";
        echo "<p><pre>";
        @ini_set('memory_limit', '1024M');
        @set_time_limit(0);
        while(($file = readdir($dh)) !== false){
          if($file != '.' && $file != '..' && $file != '.htaccess'){
            if(filetype($dir . $file) == 'file'){
              $data = file_get_contents($dir.$file);
              $rcmail->db->query(
                "INSERT INTO ".get_table_name('attachments')."
                (created, user_id, cache_key, data)
                VALUES (?, ?, ?, ?)",
                date('Y-m-d H:i:s', filemtime($dir.$file)),
                $_SESSION['user_id'],
                $file,
                base64_encode($data)
              );
              $parts = array();
              $id = $rcmail->db->insert_id(get_table_name('attachments'));
              if($id){
                @unlink($dir.$file);
              }
            }
          }
        }
        echo "</p></pre>";
        $files = scandir($dir);
        if(count($files) < 4){
          echo '<h4>Migration terminated successfully.</h4><p>Please remove "<i>dl</i>" folder and <i>$rcmail_config["dl_dir"]</i> from detach_attachments config.</p>'; 
        }
        else{
          echo '<h4>ERROR: Migration failed.</h4><p>Database import failed for the following files:<ul>';
          foreach($files as $file){
            if($file != '.' && $file != '..' && $file != '.htaccess'){
              echo '<li>' . $file . '</li>';
            }
          }
          echo '</ul>';
        }
        echo "<div><a href='./' target=_self>Back</a></div></body></html>";
      }
    }
    exit;
  }
  
  function counts(){
    $rcmail = rcmail:: get_instance();
    $hash = get_input_value('_hash', RCUBE_INPUT_POST);
    $file = urldecode(get_input_value('_file', RCUBE_INPUT_POST));
    if(md5($_SESSION['user_id']) == $hash){
      $sql_result = $rcmail->db->query(
        "SELECT downloads, user_id
        FROM ".get_table_name('attachments')."
        WHERE cache_key=?",
        $file
      );
      if($sql_arr = $rcmail->db->fetch_assoc($sql_result)){
        $count = $sql_arr['downloads'];
      }
      else{
        $count = '?';
      }
    }
    else{
      $count = '?';
    }
    $rcmail->output->command('plugin.dta_counts', $count);
  }
  
  function delete(){
    $rcmail = rcmail::get_instance();
    if($file = get_input_value('_file', RCUBE_INPUT_POST)){
      if(get_input_value('_hash', RCUBE_INPUT_POST) == md5($_SESSION['user_id'])){
        // find me: we can't check user_id due to migration ... recode when migration is removed later
        $sql_result = $rcmail->db->query(
          "DELETE FROM ".get_table_name('attachments')."
          WHERE cache_key=?",
          $file
        );
        $rcmail->output->command('plugin.dta_counts', '?');
      }
    }
  }
  
  function download(){
    if(get_input_value('_action', RCUBE_INPUT_GPC) == 'plugin.dla'){
      $rcmail = rcmail::get_instance();
      if($file = get_input_value('_dla', RCUBE_INPUT_GPC)){
        $file = urldecode($file);
        $sql_result = $rcmail->db->query(
          "SELECT data, downloads, user_id
          FROM ".get_table_name('attachments')."
          WHERE cache_key=?",
          $file
        );
        if($sql_arr = $rcmail->db->fetch_assoc($sql_result)){
          if($sql_arr['data']){
            if($delete = get_input_value('_delete', RCUBE_INPUT_GET)){
              if($delete == md5($_SESSION['user_id'])){
                // find me: we can't check user_id due to migration ... recode when migration is removed later
                $sql_result = $rcmail->db->query(
                  "DELETE FROM ".get_table_name('attachments')."
                  WHERE cache_key=?",
                  $file
                );
              }
            }
            else if($_SESSION['user_id'] != $sql_arr['user_id']){
              $downloads = $sql_arr['downloads'] + 1;
              $sql_result = $rcmail->db->query(
                "UPDATE ".get_table_name('attachments')."
                SET downloads=".$downloads."
                WHERE cache_key=?",
                $file
              );
            }
            $temp_dir = $rcmail->config->get('temp_dir');
            $tmpfname = tempnam($temp_dir, 'zip');
            $zip = new ZipArchive();
            $zip->open($tmpfname, ZIPARCHIVE::OVERWRITE);
            $zip->addFromString(preg_replace("/[^0-9A-Za-z_.]/", '_', substr($file,33)), base64_decode($sql_arr['data']));
            $zip->close();
            $browser = new rcube_browser;
            send_nocacheing_headers();
            // send download headers
            header("Content-Type: application/octet-stream");
            if($browser->ie)
              header("Content-Type: application/force-download");
            // don't kill the connection if download takes more than 30 sec.
            @set_time_limit(0);
            header("Content-Disposition: attachment; filename=\"". substr($file,33) .".zip\"");
            header("Content-length: " . filesize($tmpfname));
            readfile($tmpfname);
            @unlink($tmpfname);
          }
          else{
            $this->add_texts('localization', false);
            $rcmail->output->send('detach_attachments.error');
          }
        }
        else{
          $this->add_texts('localization', false);
          $rcmail->output->send('detach_attachments.error');
        }
        exit;
      }
    }
  }
  
  function reset($args){
    $rcmail = rcmail::get_instance();
    $rcmail->session->remove('attachments_total');
    return $args;
  }
  
  public function detach($args, $label = false){
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('cum_upload_detach') == -1){
      return $args;
    }
    $file = $args['path'];
    $total = 0;
    $id = get_input_value('_id', RCUBE_INPUT_GPC);
    if(isset($_SESSION['attachments_total'][$id])){
      $total = $_SESSION['attachments_total'][$id];
    }
    $size = filesize($file);
    $total = $total + $size;
    $_SESSION['attachments_total'][$id] = $total;
    if($total >= $rcmail->config->get('cum_upload_detach', 5) * 1024 * 1024){
      $mime = strtolower($args['mimetype']);
      if($mime == 'text/calendar' ||
         $mime == 'text/ical' ||
         $mime == 'application/ics'
      ){
        return $args;
      }
      if(!$label){
        $this->add_texts('localization', false);
        $label = $this->gettext('expires');
      }
      $args['id'] = md5(session_id() . microtime() . $_SESSION['user_id']);
      $dl = strtolower(urlencode($args['id'] . '_' . $args['name']));
      if($data = file_get_contents($file)){
        $rcmail->db->query(
          "INSERT INTO ".get_table_name('attachments')."
           (created, user_id, cache_key, data)
           VALUES (".$rcmail->db->now().", ?, ?, ?)",
          $_SESSION['user_id'],
          $dl,
          base64_encode($data)
        );

        if(isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == '1' || strtolower($_SERVER['HTTPS'])=='on'))
          $s = "s";
        else
          $s = "";
        $url = 'http' . $s . '://'.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'] . '?_action=plugin.dla&_dla=' . $dl;
        $html  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN">' . "\r\n";
        $html .= '<html><head><style type="text/css">body{font-family: "Lucida Grande", Verdana, Arial, Helvetica, sans-serif;font-size: 12px;}</style></head><body><div>';
        if($size > 1024 * 1024)
          $disp_size = round($total/1024/1024, 2) . " MBytes";
        else if($size > 1024)
          $disp_size = round($total/1024, 2) . " KBytes";
        else
          $disp_size = round($total, 2) . " Bytes";
        $html .= "<a id='" . md5($_SESSION['user_id']) . "' href='" . $url . "'>" . Q(utf8_decode(htmlentities($args['name'],ENT_COMPAT,'UTF-8'))) . "</a> (" . $disp_size . ") ";
        $html .= "[" . utf8_decode($label) . " " . date($rcmail->config->get('date_format','d.m.Y') . ' ' . $rcmail->config->get('time_format','H:i'), time() + 86400 * (int) $rcmail->config->get('dl_expires', 30)) . "]";
        $html .= "</div></body></html>";
        file_put_contents($file . '.dta', $html);
        $args['path'] = $file . '.dta';
        $args['name'] = $args['name'] . '.html';
        $args['mimetype'] = 'text/html';
      }
    }
    return $args;
  }
}
?>