<?php

/**
 * hMailServer Autoban support
 *
 *
 * @version 1.4.2 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 * Requirements: hMailServer 5.x
 *
 * Autoban needs a proper hmail configuration for webmails !!!
 * -> http://www.hmailserver.com/documentation/latest/?page=reference_autoban
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 **/
 
class hmail_autoban extends rcube_plugin
{
  public $task = 'login';
  public $noframe = true;
  public $noajax = true;
  
  private $sql_select = "SELECT * FROM hm_securityranges WHERE rangename LIKE %q";

  /* unified plugin properties */
  static private $plugin = 'hmail_autoban';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver. Please <a href="http://www.hmailserver.com/documentation/latest/?page=reference_autoban" target="_new">read</a> this!';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.4.2';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  { 
    $this->_load_config();
    $this->add_hook('authenticate', array($this, 'check_autoban'));
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  function _load_config()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
  }
  function check_autoban($args)
  {
    if(!$args['user'])
      return $args;

    $rcmail = rcmail::get_instance();
    if ($dsn = $rcmail->config->get('db_hmail_autoban_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('w');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_autoban ::: \$rcmail_config['db_hmail_autoban_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
    $sql = $this->sql_select;

    $sql = str_replace('%q', $db->quote('Auto-ban: ' . $args['user'] . '%','text'), $sql);

    $res = $db->query($sql);
                 
    if ($err = $db->is_error()){
       return;
    }
    $ret = $db->fetch_assoc($res);
    
    if($ret){
      $rcmail = rcmail::get_instance();
      $log = "HMail Autoban: " . $args['user'] . " --> IP: " . $this->getVisitorIP();
      $this->log($log);
      if(isset($args['localization'])){
        $rcmail->output->command('display_message',$args['localization'],'error');      
      }
      else{
        $this->add_texts('localization/');      
        $rcmail->output->command('display_message',rcube_label('autobanned','hmail_autoban'),'error');
      }
      $rcmail->output->send('login');
    }

    return $args;
  }

  function getVisitorIP() 
  { 
  
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 
  }
  
  function log($log)
  {
 
    $rcmail = rcmail::get_instance();
        
    if($rcmail->config->get('autoban_log')){
      write_log('autoban',$log);
    }  
  }  
  
}

?>