<?php

/**
 * Change hMailServer Autoresponder
 *
 * Plugin that gives access to Autoresponder using hMailServer database
 *
 * @version 2.9 - 29.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: (*) hMailServer 5.x
 *               (*) settings plugin @website http://myroundcube.googlecode.com
 *                   (since Roundcube 0.6 the plugin is loaded automatically)
 *
 **/ 

class hmail_autoresponder extends rcube_plugin
{
  public $task = 'mail|settings';
  
  private $sql_select = 'SELECT accountlastlogontime, accountvacationmessageon, accountvacationsubject, accountvacationmessage, accountvacationexpires, accountvacationexpiredate FROM hm_accounts WHERE accountaddress=%u;';
  private $sql_update = 'UPDATE hm_accounts SET accountlastlogontime=%t, accountvacationmessageon=%o, accountvacationsubject=%s, accountvacationmessage=%m, accountvacationexpires=%e, accountvacationexpiredate=%d WHERE accountaddress=%u;';
  
  /* unified plugin properties */
  static private $plugin = 'hmail_autoresponder';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.9';
  static private $date = '29-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->user->data['username'] == $_SESSION['username']){  
      $this->_load_config();
      $this->add_texts('localization/');
      if($rcmail->task == 'mail'){
        $this->add_hook('template_object_summary_autoresponder', array($this, 'hmail_autoresponder_form_obj'));
      }
      else{
        $this->register_action('plugin.hmail_autoresponder', array($this, 'hmail_autoresponder_init'));
        $this->register_action('plugin.hmail_autoresponder-save', array($this, 'hmail_autoresponder_save'));
        $this->register_action('plugin.hmail_autoresponder-uninstall', array($this, 'hmail_autoresponder_uninstall'));
        $this->register_handler('plugin.hmail_autoresponder_form', array($this, 'hmail_autoresponder_form'));
        $this->register_action('plugin.hmail_autoresponder_show', array($this, 'frame'));
        $this->include_script('hmail_autoresponder.js');
      }
    }
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function _load_config()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $this->require_plugin('settings');
      $this->require_plugin('jqueryui');
    }
  }

  function hmail_autoresponder_init()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->set_pagetitle($this->gettext('autoresponder')); 
    $rcmail->output->send('hmail_autoresponder.hmail_autoresponder');
  }
  
  function frame()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->add_script("rcmail.add_onload(\"rcmail.sections_list.select('accountlink')\");");
    $rcmail->output->add_script("rcmail.add_onload(\"document.frames['prefs-frame'].location.href='./?_task=settings&_action=plugin.hmail_autoresponder&_framed=1'\");");       
    $rcmail->output->send("settings");
    exit;
  }    
  
  function hmail_autoresponder_save()
  {
    $rcmail = rcmail::get_instance();
    if(strtolower($this->get_demo($_SESSION['username'])) == strtolower(sprintf($rcmail->config->get('demo_user_account'),""))){
      $rcmail->output->command('display_message', $this->gettext('demoaccount'), 'error');
      $this->hmail_autoresponder_init();
    }
    $user        = $rcmail->user->data['username'];
    $enabled     = get_input_value('_autoresponderenabled', RCUBE_INPUT_POST);
    if(!$enabled)
      $enabled = 0;
    if(isset($_SESSION['dnsblacklisted']) && $_SESSION['dnsblacklisted'] != 'pass')
      $enabled = 0;
    $subject     = get_input_value('_autorespondersubject', RCUBE_INPUT_POST);
    $body        = get_input_value('_autoresponderbody', RCUBE_INPUT_POST);
    $expires     = get_input_value('_autoresponderexpires', RCUBE_INPUT_POST);
    if(!$expires)
      $expires = 0;
    $date        = get_input_value('_autoresponderdate', RCUBE_INPUT_POST);
    if($date == ""){
      $date = "0000-00-00 00:00:00";
      $expires = 0;
    }
    else
      $date = str_replace("/","-",$date) . " 23:59:59";
            
    if (!($res = $this->_save($user,$enabled,$subject,$body,$expires,$date))) {
        if(isset($_SESSION['dnsblacklisted']) && $_SESSION['dnsblacklisted'] != 'pass'){
          $rcmail->output->command('display_message',sprintf(rcube_label('dnsblacklisted', 'hmail_autoresponder'),$_SESSION['clientip']),'error');        
        }
        else{
          $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
        }
      } else
        $rcmail->output->command('display_message', $res, 'error');
    $this->hmail_autoresponder_init();
  
  }
  
  function hmail_autoresponder_uninstall()
  {
    $rcmail = rcmail::get_instance();
    $user        = $rcmail->user->data['username'];
    $enabled     = 0;
    $subject     = '';
    $body        = '';
    $expires     = 0;
    $date = "0000-00-00 00:00:00";
    $this->_save($user,$enabled,$subject,$body,$expires,$date);
    $rcmail->output->command('plugin.plugin_manager_success', '');
  }

  function hmail_autoresponder_form_obj($args)
  {
    $rcmail = rcmail::get_instance();  
    $settings = $this->_get();
    $enabled = $settings['accountvacationmessageon'];
    $expires = $settings['accountvacationexpires'];
    $expiredate = $settings['accountvacationexpiredate'];
    
    if($enabled == 1){
      $class = "warning";
      $label_stat = $this->gettext('autoresponderisenabled');
      if($expires == 1){
        $label_stat .= '&nbsp;...&nbsp;(' . $this->gettext('autoresponderexpires') . ": " . date('r',strtotime($expiredate)) . ')';
      }
    }
    else{
      $class = "ok";
      $label_stat = $this->gettext('autoresponderdisabled');      
    }            
    
    $out  = '<fieldset><legend>&nbsp;' . $this->gettext('autoresponder') . '&nbsp;</legend>';
    $out .= '<br /><div>';
    $out .= '&nbsp;' . $label_stat . '&nbsp;...';
    $out .= $label_sens;    
    $out .= '&nbsp;[<a class= "summary' . $class . ' summaryaction" href="./?_task=settings&_action=plugin.hmail_autoresponder_show">' . $this->gettext('edit') . '</a>]';
    $out .= '</div><br />';
    $out .= '</fieldset>';
    
    $args['content'] = $out;
    return $args;
  }
  
  function hmail_autoresponder_form()
  {
    $rcmail = rcmail::get_instance();

    // add some labels to client
    $rcmail->output->add_label(
      'hmail_autoresponder.autoresponder',
      'hmail_autoresponder.dateformatinconsistency',
      'hmail_autoresponder.dateformat',
      'hmail_autoresponder.entervalidmonth',
      'hmail_autoresponder.entervalidday',
      'hmail_autoresponder.enterfordigityear',
      'hmail_autoresponder.entervaliddate',
      'hmail_autoresponder.dateinpast',
      'hmail_autoresponder.subjectempty',
      'hmail_autoresponder.and'      
    );
    
    $rcmail->output->add_script("var settings_account=true;");  

    $settings = $this->_get();
    
    $enabled     = $settings['accountvacationmessageon'];
    $subject     = $settings['accountvacationsubject'];
    $body        = $settings['accountvacationmessage'];
    $expires     = $settings['accountvacationexpires'];
    $date        = $settings['accountvacationexpiredate'];
    
    $date = str_replace("-","/",substr($date,0,10));
    
    if($date == "0000/00/00")
      $date = "";
       
    $rcmail->output->set_env('product_name', $rcmail->config->get('product_name'));
    $rcmail->output->set_env('framed', true);
    
    // return the complete edit form as table
    $user = $rcmail->user->data['username'];
    if($_SESSION['global_alias'])
      $user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('autoresponder') . ' ::: ' . $user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= "<table>\n";

    // show autoresponder properties
    $field_id = 'autoresponder_subject';
    $input_autorespondersubject = new html_textarea(array('name' => '_autorespondersubject', 'id' => $field_id, 'cols' => 48, 'rows' => 2));

    $out .= sprintf("<tr><td valign=\"top\" class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('autorespondersubject')),
                $input_autorespondersubject->show($subject));

    $field_id = 'autoresponderbody';
    $input_autoresponderbody = new html_textarea(array('name' => '_autoresponderbody', 'id' => $field_id, 'cols' => 48, 'rows' => 15));

    $out .= sprintf("<tr><td valign=\"top\" class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('autorespondermessage')),
                $input_autoresponderbody->show($body));
                
    $field_id = 'autoresponderexpires';
    $input_autoresponderexpires = new html_checkbox(array('name' => '_autoresponderexpires', 'id' => $field_id, 'value' => 1));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('autoresponderexpires')),
                $input_autoresponderexpires->show($expires?1:0));
                              
    $field_id = 'autoresponderdate';
    $input_autoresponderdate = new html_inputfield(array('name' => '_autoresponderdate', 'readonly' => 'readonly', 'id' => $field_id, 'value' => $date, 'maxlength' => 10, 'size' => 12));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('autoresponderdate')),
                $input_autoresponderdate->show($date));
               
    $field_id = 'autoresponderenabled';
    $input_autoresponderenabled = new html_checkbox(array('name' => '_autoresponderenabled', 'id' => $field_id, 'value' => 1));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('autoresponderenabled')),
                $input_autoresponderenabled->show($enabled?1:0));                                                

    $out .= "\n</table>";
    $out .= '<br />' . "\n";
    $out .= "</fieldset>\n";    

    $rcmail->output->add_gui_object('autoresponderform', 'autoresponder-form');
    
    return $out;
  }
 
  private function _get()
  {
    $rcmail = rcmail::get_instance();
      
    $sql = $this->sql_select;

    if ($dsn = $rcmail->config->get('db_hmail_autoresponder_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        {$dsn .= '?new_link=true';   
         }
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('r');

    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_autoresponder ::: \$rcmail_config['db_hmail_autoresponder_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      { return $err; }
      
    $sql = str_replace('%u', $db->quote($rcmail->user->data['username'],'text'), $sql);
   
    $res = $db->query($sql);
                 
    if ($err = $db->is_error()){
       return $err;
    }
    $ret = $db->fetch_assoc($res);
    return $ret;
  }

  private function _save($user,$enabled,$subject,$body,$expires,$date)
  {
    $cfg = rcmail::get_instance()->config;
    
    if ($dsn = $cfg->get('db_hmail_autoresponder_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$cfg->get('sql_debug'));
      $db->db_connect('w');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_autoresponder ::: \$rcmail_config['db_hmail_autoresponder_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
    $sql = $this->sql_update;
    $sql = str_replace('%t',  $db->quote(date ('Y-m-d H:i:s', time()),'text'), $sql);
    $sql = str_replace('%s',  $db->quote($subject,'text'), $sql);
    $sql = str_replace('%m',  $db->quote($body,'text'), $sql);
    $sql = str_replace('%e',  $db->quote($expires,'text'), $sql);
    $sql = str_replace('%d',  $db->quote($date,'text'), $sql);
    $sql = str_replace('%o',  $db->quote($enabled,'text'), $sql);
    $sql = str_replace('%u',  $db->quote($user,'text'), $sql);

    $res = $db->query($sql);
    
    $this->reload($user);
    
    if ($err = $db->is_error())
      return $err;
    $res = $db->affected_rows($res);
    if ($res == 0) return $this->gettext('errorsaving');
    if ($res == 1) return FALSE; // THis is the good case - 1 row updated
    return $this->gettext('internalerror');

  }
  
  private function get_demo($string)
  {
    $temparr = explode("@",$string);
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];   
  }
  
  private function reload($user)
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('hmail_client')){
      $temparr = explode("@",$user);
      $domain = $temparr[1];
      if(!class_exists('Http')){
        include INSTALL_PATH . 'plugins/hmail_autoresponder/class.http.php';
      }
      $httpConfig['method']     = 'POST';
      $httpConfig['target']     = $rcmail->config->get('hmail_client') . '?_action=reload';
      $httpConfig['timeout']    = '30';
      $httpConfig['params']     = array('_domain' => $domain, '_username' => $user, '_oldpw' => $rcmail->decrypt($_SESSION['password']));
      $http = new Http();
      $http->initialize($httpConfig);
      if(ini_get('safe_mode') || ini_get('open_basedir')){
        $http->useCurl(false);
      }
      $http->execute();
      if($http->error){
        write_log('errors', 'Plugin hmail_autoresponder: Connection failed [' . $rcmail->config->get('hmail_client') . '?_action=reload]');
        return false;
      }
      else{
        $response = $http->result;
        if($response == 'ok'){
          return true;
        }
        else{
          write_log('errors', 'Plugin hmail_autoresponder: Reload request failed [' . $user . ']');
          return false;
        }
      }
    }
  }
}

?>