<?php

/**
 * Change hMailServer Password
 *
 * Plugin that adds a possibility to change password using hMailserver COM API
 *
 * @version 2.7 - 29.10.2012
 * @author Roland 'rosali' Liebl (based on password plugin by Aleksander 'A.L.E.C' Machniak)
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: (*) hMailServer 5.x
 *               (*) settings plugin @website http://myroundcube.googlecode.com
 *                   (since Roundcube 0.6 the plugin is loaded automatically)
 * 
 **/ 

class hmail_password extends rcube_plugin
{
  public $task = 'settings';
  public $noajax = true;

  /* unified plugin properties */
  static private $plugin = 'hmail_password';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver. DCOM must be enabled. Please <a href="http://www.hmailserver.com/documentation/latest/?page=howto_dcom_permissions" target="_new">read</a> this!';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.7';
  static private $date = '29-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin')
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    if($_SESSION['username'] == $rcmail->user->data['username']){
      $this->add_texts('localization/');
      $this->_load_config();   
      $this->register_action('plugin.hmail_password', array($this, 'hmail_password_init'));
      $this->register_action('plugin.hmail_password-save', array($this, 'hmail_password_save'));
      $this->register_handler('plugin.hmail_password_form', array($this, 'hmail_password_form'));   
      $this->include_script('hmail_password.js');
    }
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  function _load_config()
  {
    $rcmail = rcmail::get_instance();
    $plugins = $rcmail->config->get('plugins');
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $this->require_plugin('settings');
    }
  }  

  function hmail_password_init()
  {
    $rcmail = rcmail::get_instance(); 
    $rcmail->output->set_pagetitle($this->gettext('changepasswd')); 
    $rcmail->output->send('hmail_password.hmail_password');
  }
  
  function hmail_password_save()
  {
    $rcmail = rcmail::get_instance();

    $confirm = $this->_confirm($rcmail->config);
    $this->add_texts('localization/');
    
    if(strtolower($this->get_demo($_SESSION['username'])) == strtolower(sprintf($rcmail->config->get('demo_user_account'),""))){
      $rcmail->output->command('display_message', $this->gettext('demoaccount'), 'error');
    }
    else if (($confirm && !isset($_POST['_curpasswd'])) || !isset($_POST['_newpasswd']))
      $rcmail->output->command('display_message', $this->gettext('nopassword'), 'error');
    else {
      $curpwd = get_input_value('_curpasswd', RCUBE_INPUT_POST);
      $newpwd = get_input_value('_newpasswd', RCUBE_INPUT_POST);
      if ($confirm && ($rcmail->decrypt($_SESSION['password']) !=  $curpwd))
        $rcmail->output->command('display_message', $this->gettext('passwordincorrect'), 'error');
      else{
        if($curpwd != $newpwd){
          if($this->_save($newpwd) == true){
            $_SESSION['password'] = $rcmail->encrypt($newpwd);
            $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
            $rcmail->plugins->exec_hook('password_change', array(
              'old_pass' => $curpwd, 'new_pass' => $newpwd));
          }
          else{
            $rcmail->output->command('display_message', $this->gettext('errorsaving'), 'error');
          }
        }
        else{
          $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
        }
      }
    }

    $this->hmail_password_init();
  }

  function hmail_password_form()
  {
    $rcmail = rcmail::get_instance();

    $confirm = $this->_confirm($rcmail->config);
    
    // add password min length to client
    $pwl = $rcmail->config->get('password_min_length');   
    if(!empty($pwl)){
      $pwl = max(6,$pwl);
    }
    else{
      $pwl = 6;
    }
        
    // add some labels to client
    $rcmail->output->add_label(
      'hmail_password.nopassword',
      'hmail_password.nocurpassword',
      'hmail_password.passwordinconsistency',
      'hmail_password.changepasswd',
      'hmail_password.passwordminlength'
    );

    $rcmail->output->add_script('var pw_min_length =' . $pwl . ';');
    $rcmail->output->add_script('var settings_account=true;'); 
    $rcmail->output->set_env('product_name', $rcmail->config->get('product_name'));
    $rcmail->output->set_env('framed', true);

    // return the complete edit form as table
    $user = $rcmail->user->data['username'];
    if($_SESSION['global_alias'])
      $user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('password') . ' ::: ' . $user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= "<table>\n";

    if ($confirm) {
      // show current password selection
      $field_id = 'curpasswd';
      $input_newpasswd = new html_passwordfield(array('name' => '_curpasswd', 'id' => $field_id, 'size' => 20));
  
      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label></td><td>%s</td></tr>\n",
                  $field_id,
                  rep_specialchars_output($this->gettext('curpasswd')),
                  $input_newpasswd->show($rcmail->config->get('curpasswd')));
    }

    // show new password selection
    $field_id = 'newpasswd';
    $input_newpasswd = new html_passwordfield(array('name' => '_newpasswd', 'id' => $field_id, 'size' => 20));
    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label></td><td>%s</td></tr>\n",
              $field_id,
              rep_specialchars_output($this->gettext('newpasswd')),
              $input_newpasswd->show($rcmail->config->get('newpasswd')));
    // show confirm password selection
    $field_id = 'confpasswd';
    $input_confpasswd = new html_passwordfield(array('name' => '_confpasswd', 'id' => $field_id, 'size' => 20));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label></td><td>%s</td></tr>\n",
              $field_id,
              rep_specialchars_output($this->gettext('confpasswd')),
              $input_confpasswd->show($rcmail->config->get('confpasswd')));

    $out .= "\n</table>";
    $out .= '<br />' . "\n";       
    $out .= "</fieldset>\n";

    $rcmail->output->add_gui_object('passform', 'password-form');
    
    return $out;
    
  }

  private function _confirm($cfg) {
    $val = $cfg->get('password_confirm_current');
    return $val;
  }

  private function _save($password)
  {
    $rcmail = rcmail::get_instance();
    $cfg = $rcmail->config;
    $username = $rcmail->user->data['username'];
    $temparr = explode("@",$username);
    $domain = $temparr[1];
    $oldpassword = $rcmail->decrypt($_SESSION['password']);
    
    if(class_exists('COM') && $cfg->get('password_change_method') != 'database'){
      try {
        $remote = $rcmail->config->get('hmailserver_remote_dcom', false);
        if($remote)
          $obApp = new COM("hMailServer.Application", $rcmail->config->get('hmailserver_server'));
        else
          $obApp = new COM("hMailServer.Application");
      }
      catch(Exception $e){
        write_log('errors', 'Plugin hmail_password: ' . $e->getMessage() . '. This problem is often caused by DCOM permissions not being set.');
        return false;
      }
      $obApp->Authenticate($username, $oldpassword);
      try{
        $obDomain = $obApp->Domains->ItemByName($domain);
        $obAccount = $obDomain->Accounts->ItemByAddress($username);
        $obAccount->Password = $password;
        $obAccount->Save();
        return true;
      }
      catch(Exception $e){
        write_log('errors', 'Plugin hmail_password: ' . $e->getMessage());
        return false;
      }
    }
    else{
      if($cfg->get('hmail_client')){
        if(!class_exists('Http')){
          include INSTALL_PATH . 'plugins/hmail_password/class.http.php';
        }
        $httpConfig['method']     = 'POST';
        $httpConfig['target']     = $cfg->get('hmail_client', './hmail_client.php') . '?_action=changepw';
        $httpConfig['timeout']    = '30';
        $httpConfig['params']     = array('_domain' => $domain, '_username' => $username, '_pw' => $password, '_oldpw' => $oldpassword);
        $http = new Http();
        $http->initialize($httpConfig);
        if(ini_get('safe_mode') || ini_get('open_basedir')){
          $http->useCurl(false);
        }
        $http->execute();
        if($http->error){
          write_log('errors', 'Plugin hmail_password: Connection failed [' . $cfg->get('hmail_client', './') . '?_action=changepw]');
          return false;
        }
        else{
          $response = $http->result;
          if($response == 'ok'){
            return true;
          }
          else{
            write_log('errors', 'Plugin hmail_password: Change password request failed [' . $username . ']');
            return false;
          }
        }
      }
      else{
        $salt = substr(md5(time()),0,6);
        $password = hash('sha256', $salt . $password);
        $password = $salt . $password;
        if ($dsn = $cfg->get('db_hmail_accounts_dsn')) {
          // #1486067: enable new_link option
          if (is_array($dsn) && empty($dsn['new_link']))
            $dsn['new_link'] = true;
          else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
            $dsn .= '?new_link=true';
          $db = new rcube_mdb2($dsn, '', FALSE);
          $db->set_debug((bool)$cfg->get('sql_debug'));
          $db->db_connect('w');
        } else {
          die("FATAL ERROR ::: RoundCube Plugin ::: hmail_password ::: \$rcmail_config['db_hmail_password_dsn'] undefined !!! ==> die");
        }
        if ($err = $db->is_error())
          return $err;

        $res = $db->query("UPDATE hm_accounts SET accountpassword=?, accountpwencryption=? WHERE accountaddress=?", $password, 3, strtolower($username));
    
        if ($err = $db->is_error())
          return $err;
      
        $res = $db->affected_rows($res);
        if ($res == 0) {
          write_log('errors', 'Plugin hmail_password: Change password request failed [' . $username . ']');
          return false;
        }
        if ($res == 1) {
          $this->reload($username);
          return true; // THis is the good case - 1 row updated
        }
        write_log('errors', 'Plugin hmail_password: Change password request failed [' . $username . ']');
        return $this->gettext('internalerror');
      }
    }
  }
  
  private function reload($user)
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('hmail_client')){
      $temparr = explode("@",$user);
      $domain = $temparr[1];
      if(!class_exists('Http')){
        include INSTALL_PATH . 'plugins/hmail_password/class.http.php';
      }
      $httpConfig['method']     = 'POST';
      $httpConfig['target']     = $rcmail->config->get('hmail_client') . '?_action=reload';
      $httpConfig['timeout']    = '30';
      $httpConfig['params']     = array('_domain' => $domain, '_username' => $user, '_oldpw' => $rcmail->decrypt($_SESSION['password']));
      $http = new Http();
      $http->initialize($httpConfig);
      if(ini_get('safe_mode')){
        $http->useCurl(false);
      }
      $http->execute();
      if($http->error){
        write_log('errors', 'Plugin hmail_password: Connection failed [' . $rcmail->config->get('hmail_client') . '?_action=reload]');
        return false;
      }
      else{
        $response = $http->result;
        if($response == 'ok'){
          return true;
        }
        else{
          write_log('errors', 'Plugin hmail_password: Reload request failed [' . $user . ']');
          return false;
        }
      }
    }
  }
  
  private function get_demo($string)
  {
    $temparr = explode("@",$string);
  
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];   
  }
}
?>