<?php

/**
 * Change hMailServer Signature
 *
 * Plugin that gives access to Signature using hMailServer database
 *
 * @version 2.4 - 29.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: (*) hMailServer 5.x
 *               (*) settings plugin @website http://myroundcube.googlecode.com
 *                   (since Roundcube 0.6 the plugin is loaded automatically)
 * 
 **/ 

class hmail_signature extends rcube_plugin
{
  public $task = 'settings';
  
  private $sql_select = 'SELECT * FROM hm_accounts WHERE accountaddress=%u';
  private $sql_update = 'UPDATE hm_accounts SET accountlastlogontime=%d, accountenablesignature=%o, accountsignatureplaintext=%t, accountsignaturehtml=%h WHERE accountaddress=%u';
  private $sql_rcselect = 'SELECT * FROM %identities WHERE email=%u AND user_id=%id;';
  private $sql_rciidselect = 'SELECT * FROM %identities WHERE identity_id=%iid AND user_id=%id;';  
  private $sql_rcupdate = 'UPDATE %identities SET signature=NULL, html_signature=\'0\' WHERE email=%u AND user_id=%id;';
  
  /* unified plugin properties */
  static private $plugin = 'hmail_signature';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver.';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.4';
  static private $date = '29-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';


  function init()
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->user->data['username'] == $_SESSION['username']){
      $this->add_texts('localization/');
      $this->_load_config();
      $this->register_action('plugin.hmail_signature', array($this, 'hmail_signature_init'));
      $this->register_action('plugin.hmail_signature-save', array($this, 'hmail_signature_save'));
      $this->register_action('plugin.hmail_signature-uninstall', array($this, 'hmail_signature_uninstall'));
      $this->register_action('plugin.hmail_signature_show', array($this, 'frame'));
      $this->register_handler('plugin.hmail_signature_form', array($this, 'hmail_signature_form'));
      $this->include_script('hmail_signature.js');
      $this->add_hook('template_object_identityform', array($this, 'hide_signature'));
      $_SESSION['tinymcelang'] = substr($_SESSION['language'], 0, 2);
      if (!file_exists('program/js/tiny_mce/langs/'.$_SESSION['tinymcelang'].'.js')) {
        $_SESSION['tinymcelang'] = 'en';
      }
    }
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function _load_config()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $this->require_plugin('settings');
    }
  }
  
  function frame()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->add_script('rcmail.sections_list.select("accountlink")', 'docready');
    if($rcmail->config->get('skin', 'classic') == 'larry'){
      $rcmail->output->add_script('window.setTimeout(function(){$("#preferences-frame").attr("src", "./?_task=settings&_action=plugin.hmail_signature&_framed=1")}, 500)', 'docready');
    }
    else{
      $rcmail->output->add_script('window.setTimeout(function(){$("#prefs-frame").attr("src", "./?_task=settings&_action=plugin.hmail_signature&_framed=1")}, 500)', 'docready');
    }
    $rcmail->output->send("settings");
    exit;
  }  
  
  function hmail_signature_init()
  {

    $rcmail = rcmail::get_instance();
    $rcmail->output->set_pagetitle($this->gettext('signature')); 
    $rcmail->output->send('hmail_signature.hmail_signature');
    
  }
  
  function hmail_signature_save()
  {
    $this->add_texts('localization/');  
    $rcmail = rcmail::get_instance();  
    $user        = $rcmail->user->data['username'];
    $id          = $rcmail->user->data['user_id'];
    $enabled     = get_input_value('_signatureenabled', RCUBE_INPUT_POST);
    if(!$enabled)
      $enabled = 0;
    $plaintext   = get_input_value('_signatureplaintext', RCUBE_INPUT_POST);

    $html = trim($_POST['_signaturehtml']);

    if (!($res = $this->_save($user,$enabled,$plaintext,$html))) {
        $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
      } else {
        $rcmail->output->command('display_message', $res, 'error');
      }
    if(!($enabled == 1 && ($plaintext != "" || $html != "")))
      $enabled = 0;      
    
    if ($res = $this->_syncrc($user, $id, $enabled))
      $rcmail->output->command('display_message', $res, 'error');

    $this->hmail_signature_init();
  
  }

  function hmail_signature_form()
  {
    $rcmail = rcmail::get_instance();

    // add some labels to client
    $rcmail->output->add_label(
      'hmail_signature.signature'
    );
    
    $rcmail->output->add_script("var settings_account=true;");
      
    $user        = $rcmail->user->data['username'];
    $id          = $rcmail->user->data['user_id'];
        
    $settings = $this->_get($user,$id);
    
    $enabled     = $settings['accountenablesignature'];
    $plaintext   = $settings['accountsignatureplaintext'];
    $html        = $settings['accountsignaturehtml'];

    $rc_signature = html_entity_decode($settings['signature']);
    if(function_exists("utf8_encode"))
      $rc_signature = utf8_encode($rc_signature);
       
    if($settings['signature'] != "" && $plaintext == ""){
      $txt = new html2text($rc_signature, false, true);      
      $plaintext = $txt->get_text();
    }

    if($settings['signature'] != "" && $html == ""){
      $html = $settings['signature'];
    }
                   
    $rcmail->output->set_env('product_name', $rcmail->config->get('product_name'));
    $rcmail->output->set_env('framed', true);

    // return the complete edit form as table
    $display_user = $user;
    if($_SESSION['global_alias'])
      $display_user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('signature') . ' ::: ' . $display_user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= "<table>\n";

    // show signature properties
    $field_id = 'signatureplaintext';
    $input_signatureplaintext = new html_textarea(array('name' => '_signatureplaintext', 'id' => $field_id, 'cols' => 78, 'rows' => 14));

    $out .= sprintf("<tr><td colspan=\"2\" valign=\"top\" class=\"title\"><label for=\"%s\">%s</label>:</td></tr></tr><td colspan=\"2\">%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('signatureplaintext')),
                $input_signatureplaintext->show($plaintext));
                
    $field_id = 'signaturehtml';
    $input_signatureplaintext = new html_textarea(array('name' => '_signaturehtml', 'id' => $field_id, 'cols' => 78, 'rows' => 14, 'class' => 'mce_editor'));

    $out .= "<tr><td colspan=\"2\">&nbsp;</td></tr>\n";

    $out .= sprintf("<tr><td colspan=\"2\" valign=\"top\" class=\"title\"><label for=\"%s\">%s</label>:</td></tr></tr><td colspan=\"2\">%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('signaturehtml')),
                $input_signatureplaintext->show($html));
                
    $out .= "<tr><td colspan=\"2\">&nbsp;</td></tr>\n";
                    
    $field_id = 'signatureenabled';
    $input_signatureenabled = new html_checkbox(array('name' => '_signatureenabled', 'id' => $field_id, 'value' => 1));
    $out .= sprintf("<tr><td colspan=\"2\" class=\"title\"><label for=\"%s\">%s</label>:&nbsp;%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('signatureenabled')),
                $input_signatureenabled->show($enabled?1:0));

    $out .= "\n</table>";
    $out .= '<br />' . "\n";
    $out .= "</fieldset>\n";
    
    $out .= '<script type="text/javascript" src="program/js/tiny_mce/tiny_mce.js"></script>' . "\n";
    $out .= '<script type="text/javascript" src="program/js/editor.js"></script>' . "\n";
    if(class_exists('tinymce'))
      $out .= '<script type="text/javascript" src="plugins/tinymce/editor_init.js"></script>' . "\n";

    $rcmail->output->add_gui_object('autoresponderform', 'autoresponder-form');

    if(strpos($lang, 'zh_') === 0)
      $lang = str_replace('_', '-', $lang);
    else
      $lang = substr($lang, 0, 2);

    if(!file_exists(INSTALL_PATH . 'program/js/tiny_mce/langs/'.$lang.'.js'))
      $lang = 'en';

    $rcmail->output->add_script(sprintf("rcmail_editor_init(%s)",
      json_encode(array(
        'mode'       => 'identity',
        'lang'       => $lang,
        'skin_path'  => $rcmail->output->get_skin_path(),
        'spellcheck' => intval($rcmail->config->get('enable_spellcheck')),
        'spelldict'  => intval($rcmail->config->get('spellcheck_dictionary')),
      ))), 'docready');

    return $out;
  }
 
  private function _get($user, $id)
  {
    $rcmail = rcmail::get_instance();
      
    $sql = $this->sql_rcselect;
    $dsn = $rcmail->config->get('db_dsnw');
    $db = new rcube_mdb2($dsn, '', FALSE);
    $sql = str_replace('%identities', get_table_name('identities'), $sql); 
    $sql = str_replace('%u', $db->quote($user,'text'), $sql);
    $sql = str_replace('%id', $db->quote($id,'text'), $sql);
  
    $res = $db->query($sql);
    if ($err = $db->is_error()){
       return $err;
    }
    $rcret = $db->fetch_assoc($res);
    if(!is_array($rcret))
      $rcret = array();
      
    $sql = $this->sql_select;

    if ($dsn = $rcmail->config->get('db_hmail_signature_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('r');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_signature ::: \$rcmail_config['db_hmail_signature_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
      
    $sql = str_replace('%u', $db->quote($rcmail->user->data['username'],'text'), $sql);
    $res = $db->query($sql);
                 
    if ($err = $db->is_error()){
       return $err;
    }
    $ret = $db->fetch_assoc($res);
    if(!is_array($ret))
      $ret = array();

    return array_merge($rcret,$ret);  
  }
  
  function hmail_signature_uninstall()
  {
    $rcmail = rcmail::get_instance();
    $user        = $rcmail->user->data['username'];
    $enabled     = 0;
    $plaintext   = '';
    $html        = '';
    $this->_save($user,$enabled,$plaintext,$html);
    $rcmail->output->command('plugin.plugin_manager_success', '');
  }

  private function _save($user,$enabled,$plaintext,$html)
  {
    $cfg = rcmail::get_instance()->config;

    $sql = $this->sql;
    
    if ($dsn = $cfg->get('db_hmail_signature_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$cfg->get('sql_debug'));
      $db->db_connect('w');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_signature ::: \$rcmail_config['db_hmail_signature_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
    $sql = $this->sql_update;
    $sql = str_replace('%d',  $db->quote(date ('Y-m-d H:i:s', time()),'text'), $sql);  
    $sql = str_replace('%o',  $db->quote($enabled,'text'), $sql);
    $sql = str_replace('%t',  $db->quote($plaintext,'text'), $sql);
    $sql = str_replace('%h',  $db->quote($html,'text'), $sql);
    $sql = str_replace('%u',  $db->quote($user,'text'), $sql);
    
    $res = $db->query($sql);
    
    $this->reload($user);
   
    if ($err = $db->is_error())
      return $err;
    $res = $db->affected_rows($res);
    if ($res == 0) return $this->gettext('errorsaving');
    if ($res == 1) return FALSE; // THis is the good case - 1 row updated
    return $this->gettext('internalerror');

  }
  
  private function _syncrc($user,$id,$enabled){
  
    if($enabled != 1)
      return FALSE;
  
    $cfg = rcmail::get_instance()->config;
    
    $db = new rcube_mdb2($cfg->get('db_dsnw'), '', FALSE);
    $db->set_debug((bool)$cfg->get('sql_debug'));
    $db->db_connect('w');
 
    if ($err = $db->is_error())
      return $err;
      
    $sql = $this->sql_rcupdate;
    $sql = str_replace('%identities', get_table_name('identities'), $sql);
    $sql = str_replace('%u', $db->quote($user,'text'), $sql);
    $sql = str_replace('%id', $db->quote($id,'text'), $sql);
    
    $res = $db->query($sql);
    
    if ($err = $db->is_error()) return $err;
    $res = $db->affected_rows($res);
    if ($res >= 0) return FALSE; // This is the good case
    return $this->gettext('internalerror');
  
  }
  
  function hide_signature($p){

    $rcmail = rcmail::get_instance();
    $this->add_texts('localization/');

    $stat = $this->_get($rcmail->user->data['username'],$rcmail->user->data['user_id']);
    if($stat['accountenablesignature'] == "0")
      return $p;
    
    $db = new rcube_mdb2($rcmail->config->get('db_dsnw'), '', FALSE);
    $db->set_debug((bool)$rcmail->config->get('sql_debug'));
    $db->db_connect('w');

    if ($err = $db->is_error())
      return $err;
      
    $sql = $this->sql_rciidselect;
    
    $user = $rcmail->user->data['username'];
    $id   = $rcmail->user->data['user_id']; 
    $iid  = get_input_value('_iid', RCUBE_INPUT_GET);  
    
    $sql = str_replace('%identities', get_table_name('identities'), $sql);
    $sql = str_replace('%iid', $db->quote($iid,'text'), $sql);
    $sql = str_replace('%id', $db->quote($id,'text'), $sql);
    $res = $db->query($sql);
    $ret = $db->fetch_assoc($res);
    
    if(strtolower($user) != strtolower($ret['email']))   
      return $p;
    
    $in = $p['content'];
    $navout = './?_task=settings&_action=plugin.hmail_signature_show';
    $temparr = explode("<tr>",$in);
    $skin =  $rcmail->config->get('skin');
    $img = "plugins/hmail_signature/skins/" . $skin . "/images/signature.png";
    if(!file_exists($skin))
      $img = "plugins/hmail_signature/skins/classic/images/signature.png";
    $temparr[6] = '<tr><td class="title"><label for="rcmfd_signature">' . $this->gettext('signature') . '</label></td><td><img alt="' . $this->gettext('signature') . '" title="' . $this->gettext('signature') . '" src="' . $img . '" onclick="parent.document.location.href=\'' . $navout . '\'" /></td></tr>' . "\n";
    unset($temparr[7]);
    
    $out = "";
    foreach($temparr as $key => $val)
      $out .= "<tr>\n" . $val;

    $p['content'] = $out;

    return $p;  
  }
  
  private function reload($user)
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('hmail_client')){
      $temparr = explode("@",$user);
      $domain = $temparr[1];
      if(!class_exists('Http')){
        include INSTALL_PATH . 'plugins/hmail_signature/class.http.php';
      }
      $httpConfig['method']     = 'POST';
      $httpConfig['target']     = $rcmail->config->get('hmail_client') . '?_action=reload';
      $httpConfig['timeout']    = '30';
      $httpConfig['params']     = array('_domain' => $domain, '_username' => $user, '_oldpw' => $rcmail->decrypt($_SESSION['password']));
      $http = new Http();
      $http->initialize($httpConfig);
      if(ini_get('safe_mode') || ini_get('open_basedir')){
        $http->useCurl(false);
      }
      $http->execute();
      if($http->error){
        write_log('errors', 'Plugin hmail_signature: Connection failed [' . $rcmail->config->get('hmail_client') . '?_action=reload]');
        return false;
      }
      else{
        $response = $http->result;
        if($response == 'ok'){
          return true;
        }
        else{
          write_log('errors', 'Plugin hmail_signature: Reload request failed [' . $user . ']');
          return false;
        }
      }
    }
  }
}

?>