<?php

/**
 * nabble
 *
 * @version 2.8 - 14.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/ 
 
class nabble extends rcube_plugin
{
  public $noajax = true;
  
  private $user = false;
  private $pass = false;
  private $error = false;
  
  /* unified plugin properties */
  static private $plugin = 'nabble';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.8';
  static private $date = '14-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = array('use_nabble', 'nabbleuser', 'nabblefid', 'nabble_maillist_name', 'nabblepass');
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $this->add_texts('localization/');
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins', array()))){
      $this->load_config();
    }
    $this->add_hook('startup', array($this, 'nabble_show'));
    $this->add_hook('startup', array($this, 'outside'));
    $this->add_hook('template_object_nabble_global', array($this, 'nabble_global'));
    $this->add_hook('template_object_nabble_user', array($this, 'nabble_user'));
    $this->add_hook('template_object_nabble_embed', array($this, 'nabble_embed'));
    $this->add_hook('template_object_nabble_outside', array($this, 'nabble_outside'));
    $this->add_hook('preferences_list', array($this, 'settings_table'));
    $this->add_hook('preferences_save', array($this, 'save_prefs'));
    $this->add_hook('preferences_sections_list', array($this, 'nabbleLink'));
    $this->register_action('plugin.nabble_login', array($this, 'login')); 
    $this->register_action('plugin.nabble_embed', array($this, 'embed'));
    $this->register_action('plugin.nabble_prefs', array($this, 'navigation'));
    
    $rcmail = rcmail::get_instance();
    $this->user = $rcmail->config->get('nabbleuser');
    $this->pass = $rcmail->config->get('nabblepass');
    $this->pass = $rcmail->decrypt($this->pass);
     
    $skin  = $rcmail->config->get('skin','classic');
    
    $rcmail->output->set_env('nabble_skin', $skin);
    
    if(!file_exists("plugins/nabble/skins/$skin/nabble.css"))
      $skin = "default";

    $this->include_stylesheet('skins/' . $skin . '/nabble.css');
    
    if($rcmail->config->get('use_nabble')){
      $this->include_script('nabble.js');
    }
    // add taskbar button
    $token = '';
    if(in_array('compressor', $rcmail->config->get('plugins', array()))){
      $token = '&_s=' . md5($_SESSION['language'] . 
          session_id() .
          $rcmail->config->get('skin', 'classic') . 
          self::$version . 
          self::$date . 
          serialize($rcmail->config->get('plugins', array())) .
          serialize($rcmail->config->get('plugin_manager_active', array()))
         );
    }
    $this->add_button(array(
      'name'    => 'nabble',
      'command' => 'plugin.nabble',
      'class'   => 'button-nabble',
      'id'      => 'nabble_button',
      'content' => html::tag('span', array('class' => 'button-inner'), $this->gettext('nabble.nabbleapps')),
      'href'    => './?_task=dummy&_action=plugin.nabble' . $token,
      'style'   => 'display:none;',
      ), 'taskbar');
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function login()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->send("nabble.login");
  }

  function embed()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->send("nabble.embed");
  }
  
  function outside()
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->action == "plugin.nabble_outside")
      $rcmail->output->send("nabble.outside");
  }
  
  function nabbleLink($args)
  {
    $temp = $args['list']['server'];
    unset($args['list']['server']);
    $args['list']['nabblelink']['id'] = 'nabblelink';
    $args['list']['nabblelink']['section'] = $this->gettext('nabbleapps');
    $args['list']['server'] = $temp;

    return $args;
  }   
  
  function settings_table($args)
  {
    if ($args['section'] == 'nabblelink') {
      $rcmail = rcmail::get_instance();    
      $use_nabble = $rcmail->config->get('use_nabble');
      $field_id = 'rcmfd_use_nabble';
      $checkbox = new html_checkbox(array('name' => '_use_nabble', 'id' => $field_id, 'value' => 1));
      $args['blocks']['nabble']['name'] = $this->gettext('nabbleapps');
      $args['blocks']['nabble']['options']['use_nabble'] = array(
        'title' => html::label($field_id, Q($this->gettext('usenabble'))),
        'content' => $checkbox->show($use_nabble?1:0),
      );

      $field_id = 'rcmfd_nabble_user';
      $input_nabbleuser = new html_inputfield(array('name' => '_nabbleuser', 'id' => $field_id, 'size' => 35, 'autocomplete' => 'off'));
      $args['blocks']['nabble']['options']['nabbleuser'] = array(
        'title' => html::label($field_id, Q($this->gettext('nabbleuser'))),
        'content' => $input_nabbleuser->show($rcmail->config->get('nabbleuser')),
      );
      
      $field_id = 'rcmfd_nabble_pass';
      if($rcmail->config->get('nabblepass'))
        $title = $this->gettext('nabblepassisset');
      else
        $title = $this->gettext('nabblepassisnotset');
      $input_nabblepass = new html_passwordfield(array('name' => '_nabblepass', 'id' => $field_id, 'size' => 35, 'title' => $title, 'autocomplete' => 'off'));
      $args['blocks']['nabble']['options']['nabblepass'] = array(
        'title' => html::label($field_id, Q($this->gettext('nabblepass'))),
        'content' => $input_nabblepass->show(),
      );
      
      $field_id = 'rcmfd_nabble_name';
      $input_nabblename = new html_inputfield(array('name' => '_nabblename', 'id' => $field_id, 'size' => 35, 'autocomplete' => 'off'));
      $args['blocks']['nabble']['options']['nabblename'] = array(
        'title' => html::label($field_id, Q($this->gettext('nabblename'))),
        'content' => $input_nabblename->show($rcmail->config->get('nabble_maillist_name')),
      );

      $field_id = 'rcmfd_nabble_fid';
      $input_nabblefid = new html_inputfield(array('name' => '_nabblefid', 'id' => $field_id, 'size' => 10, 'autocomplete' => 'off'));
      $args['blocks']['nabble']['options']['nabblefid'] = array(
        'title' => html::label($field_id, Q($this->gettext('nabblefid'))),
        'content' => $input_nabblefid->show($rcmail->config->get('nabblefid')),
      );
    }
    return $args;
  }
    
  function save_prefs($args)
  {
    if ($args['section'] == 'nabblelink') {
      $rcmail = rcmail::get_instance();
      $args['prefs']['use_nabble'] = isset($_POST['_use_nabble']) ? true : false;
      $args['prefs']['nabbleuser'] = get_input_value('_nabbleuser', RCUBE_INPUT_POST);
      $args['prefs']['nabblefid'] = get_input_value('_nabblefid', RCUBE_INPUT_POST);
      $args['prefs']['nabble_maillist_name'] = get_input_value('_nabblename', RCUBE_INPUT_POST);
      $pass = get_input_value('_nabblepass', RCUBE_INPUT_POST);
      if($pass){
        $args['prefs']['nabblepass'] = $rcmail->encrypt($pass);
      }
      $rcmail->output->add_script("parent.location.href='./?_task=settings&_action=plugin.nabble_prefs';");
    }
    return $args;
  }
  
  function navigation(){
    rcmail::get_instance()->output->add_script("$(document).ready(function(){ $('#prefs-frame').attr('src','./?_task=settings&_action=edit-prefs&_section=nabblelink&_framed=1'); })", 'foot');
  }
  
  function nabble_show($args)
  {  
    $rcmail = rcmail::get_instance();
    if($rcmail->action == "plugin.nabble"){
      if(empty($_SESSION['username']))  
        $rcmail->output->send("nabble.nabble");
      else{
        $rcmail->output->send("nabble.account");
      }
    }
    return $args;
  }
  
  function nabble_global($args)
  {
    $rcmail = rcmail::get_instance();
    $id = $rcmail->config->get('global_nabble_maillist_id');
    $temparr = explode('.', $id);
    $name = $rcmail->config->get('global_nabble_maillist_name');
    $link = "http://$name.$id.nabble.com/";
    $args['content'] = '
      <div style="position:absolute;top:0px;bottom:0px;left:0px;right:0px;overflow:auto;">
      <a id="nabblelink" href="' . $link . '">'.$this->gettext('nabblelink').'</a>
       <script src="http://' . $name . '.' . $id . '.nabble.com/embed/f'.$temparr[0].'"></script>
      </div>
    ';    
    return $args;
  }

  function nabble_embed($args)
  {
    $rcmail = rcmail::get_instance();
    $name = $rcmail->config->get('nabble_maillist_name');
    $id = $rcmail->config->get('nabblefid');
    if(empty($id))
      $id = "40634.n3";
    $temparr = explode('.', $id);
    $link = "http://$name.$id.nabble.com/";
    $args['content'] = '
      <div style="position:absolute;top:0px;bottom:0px;left:0px;right:0px;overflow:auto;">    
      <a id="nabblelink" href="' . $link . '">Nabble</a>
      <script src="http://' . $name . '.' . $id . '.nabble.com/embed/f'.$temparr[0].'"></script>
      </div>
    ';
    return $args;
  }
  
  function nabble_user($args)
  {
    $rcmail = rcmail::get_instance();
    $user = $this->user;
    $pass = $this->pass;
    if(isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == '1' || strtolower($_SERVER['HTTPS'])=='on'))
      $s = "s";
    else
      $s = "";
    $id = $rcmail->config->get('nabblefid');
    $name = $rcmail->config->get('nabble_maillist_name');
    //$url = 'http' . $s . '://'.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'] . '?_action=plugin.nabble_embed';
    $url = '/';
    $content = '
<form method="post" name="f" action="http://' . $name . '.' . $id .'.nabble.com/template/NamlServlet.jtp?login_page">
  <input type="hidden" name="macro" value="login_page">
  <input type="hidden" name="nextUrl" value="' . $url . '">
  <input type="hidden" name="email" value="' . $user . '">
  <input type="hidden" name="password" value="' . $pass . '">
</form>
<script type="text/javascript">
  document.forms.f.submit();
</script>
';
    $args['content'] = $content;    
    return $args;
  }
  
  function nabble_outside($args)
  {
    $rcmail = rcmail::get_instance();
    $id = $rcmail->config->get('nabblefid');
    $name = $rcmail->config->get('nabble_maillist_name');
    if(empty($id))
      $id = "40634.n3";
    $temparr = explode('.', $id);
    $link = "http://$name.$id.nabble.com/";
    $args['content'] = '
      <a id="nabblelink" href="' . $link . '">Nabble</a>
      <script src="http://' . $name . '.' . $id . '.nabble.com/embed/f'.$temparr[0].'"></script>
    ';

    return $args;  
  } 

}
?>