<?php
/**
 * newuser plugin
 *
 * Currently this plugin sends a welcome message to new users
 * and informs the admin that a new user was created (both optional).
 *
 * @version 1.8.2 - 05.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/

/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/ 

class newuser extends rcube_plugin
{

  public static $lang = "en_US";
  public $task = 'login';
  public $noframe = true;
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'newuser';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.8.2';
  static private $date = '05-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    if(is_dir("./plugins/newuser/localization/" . $_SESSION['language'])){
      newuser::$lang = $_SESSION['language'];
    }
    $this->add_hook('user_create', array($this, 'newuser_jobs'));
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function newuser_jobs($args)
  {
    $args = $this->newuser_inform_admin($args);
    $demo = rcmail::get_instance()->config->get('newuser_ignore_demo_accounts');
    $demo = sprintf($demo, "");
    if(strtolower($demo) != strtolower($this->get_demo($user_email))){
      $args = $this->newuser_send_welcome($args);
    }
    return $args;
  }
  
  function newuser_inform_admin($args)
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    if($rcmail->config->get('newuser_inform_admin')){
      $user_email = $args['user'];
      if(!empty($args['user_email']))
        $user_email = $args['user_email'];
      if($rcmail->config->get('newuser_ignore_demo_accounts')){
        $demo = $rcmail->config->get('newuser_ignore_demo_accounts');
        $demo = sprintf($demo, "");
        if(strtolower($demo) == strtolower($this->get_demo($user_email))){
          return $args;
        }
      }
      $subject_pattern = $this->newuser_get_pattern('inform_admin_subject.txt',$this->lang);
      $body_pattern    = $this->newuser_get_pattern('inform_admin_body.html',$this->lang);
      $footer = $rcmail->config->get('newuser_message_footer','Mail4Us - Webm@il Team');
       
      if($body_pattern && $subject_pattern){  
        $subject = sprintf( 
                    $subject_pattern, 
                    $args['user'],
                    $this->getVisitorIP()
                    );
                      
        $body = sprintf( 
                    $body_pattern, 
                    $args['user'],
                    $this->getVisitorIP(),
                    $footer
                    );
          
        $from = $user_email;
        $to = $rcmail->config->get('newuser_inform_admin');
        $this->sendmessage($from,$to,$subject,$body);
      }            
    }     
    return $args;
  }
  
  function newuser_send_welcome($args)
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('newuser_inform_admin')){
      $isHTML = 1;
      $subject_pattern = newuser::newuser_get_pattern('welcome_subject.txt',newuser::$lang);
      $body_pattern    = newuser::newuser_get_pattern('welcome_body.html',newuser::$lang);
      $footer = $rcmail->config->get('newuser_message_footer','Mail4Us - Webm@il Team');
      $user_email = $args['user'];
      if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == TRUE) 
        $s = "s"; 
      $roundcube_url = "http$s://" . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'] . "?_task=logout"; 
    
      $link  = '<a target="_new" href="' . $roundcube_url . '">'; 
      $link .= $roundcube_url; 
      $link .= '</a>';
       
      if(!empty($args['user_email']))
        $user_email = $args['user_email'];
       
      if($body_pattern && $subject_pattern){  
        $subject = sprintf( 
                    $subject_pattern, 
                    $rcmail->config->get('product_name')
                    );
                    
        $body = sprintf( 
                    $body_pattern, 
                    $args['user'],
                    $link,
                    $footer
                    );
                    
        $from = $rcmail->config->get('newuser_inform_admin');
        $to = $user_email;
        newuser::sendmessage($from,$to,$subject,$body);
      }
    }
    return $args;
  }
  
  function newuser_get_pattern($pattern,$lang)
  {
    if(file_exists("./plugins/newuser/localization/" . $lang . "/" . $pattern)){ 
      $pattern = file_get_contents("./plugins/newuser/localization/" . $lang . "/" . $pattern); 
    }
    if(file_exists("./plugins/newuser/localization/" . $lang . "/" . $pattern . ".dist")){ 
      $pattern = file_get_contents("./plugins/newuser/localization/" . $lang . "/" . $pattern . ".dist"); 
    } 
    else if(file_exists("./plugins/newuser/localization/en_US/" . $pattern)){ 
      $pattern = file_get_contents("./plugins/newuser/localization/en_US/" . $pattern); 
    } 
    else{ 
      $pattern = file_get_contents("./plugins/newuser/localization/en_US/" . $pattern . ".dist"); 
    }   
    
    return $pattern;
    
  }
  
  function sendmessage($from,$to,$subject,$body)
  {
    $rcmail = rcmail::get_instance();  
    $ctb = md5(rand() . microtime());
    
    $headers  = "Return-Path: $from\r\n";
    $headers .= "MIME-Version: 1.0\r\n";
    $headers .= "Content-Type: multipart/alternative; boundary=\"=_$ctb\"\r\n";
    $headers .= "X-Domain: " . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'] . "\r\n";
    $headers .= "X-Server-Admin: " . $_SERVER['SERVER_ADMIN'] . "\r\n";
    $headers .= "Date: " . date('r', time()) . "\r\n";
    $headers .= "From: $from\r\n";
    $headers .= "To: $to\r\n";
    $headers .= "Subject: " . '=?UTF-8?B?'.base64_encode($subject).'?=' . "\r\n";
    $headers .= "Reply-To: $from\r\n";

    $msg_body .= "Content-Type: multipart/alternative; boundary=\"=_$ctb\"\r\n\r\n";

    $txt_body  = "--=_$ctb";
    $txt_body .= "\r\n";
    $txt_body .= "Content-Transfer-Encoding: 7bit\r\n";
    $txt_body .= "Content-Type: text/plain; charset=" . RCMAIL_CHARSET . "\r\n";
    $LINE_LENGTH = $rcmail->config->get('line_length', 72);
    $h2t = new html2text($body, false, true, 0);
    $txt = rc_wordwrap($h2t->get_text(), $LINE_LENGTH, "\r\n");
    $txt = wordwrap($txt, 998, "\r\n", true);
    $txt_body .= "$txt\r\n";
    $txt_body .= "--=_$ctb";
    $txt_body .= "\r\n";
          
    $msg_body .= $txt_body;
          
    $msg_body .= "Content-Transfer-Encoding: quoted-printable\r\n";
    $msg_body .= "Content-Type: text/html; charset=" . RCMAIL_CHARSET . "\r\n\r\n";
    $msg_body .= str_replace("=","=3D",$body);
    $msg_body .= "\r\n\r\n";
    $msg_body .= "--=_$ctb--";
    $msg_body .= "\r\n\r\n";
                
    // send message         
    if (!is_object($rcmail->smtp))
      $rcmail->smtp_init(true);
        
    if($rcmail->config->get('smtp_pass') == "%p"){
      $rcmail->config->set('smtp_server', $rcmail->config->get('default_smtp_server'));
      $rcmail->config->set('smtp_user', $rcmail->config->get('default_smtp_user'));
      $rcmail->config->set('smtp_pass', $rcmail->config->get('default_smtp_pass'));
    }          
    $rcmail->smtp->connect();         
    $rc = $rcmail->smtp->send_mail($from, $to, $headers, $msg_body);
    return $rc;   
  }
  
  function getVisitorIP()
  { 
  
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 
  }

  function get_demo($string){

    $temparr = explode("@",$string);
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];

  }

}

?>