<?php
/**
 * Plaxo Contacts
 *
 * @version 2.2.2 - 18.10.2012
 * @author Roland 'rosali' Liebl (based on automatic_addressbook by Jocelyn Delalande)
 * @website http://myroundcube.googleocode.com
 * @licence GNU GPL
 * 
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 **/    
 
class plaxo_contacts extends rcube_plugin
{

  public  $task = "mail|addressbook|settings|dummy";
  private $abook_id = 'plaxo_contacts';
  private $user = false;
  private $pass = false;
  private $contacts;
  private $error = false;
  private $results = null;
  
  /* unified plugin properties */
  static private $plugin = 'plaxo_contacts';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.2.2';
  static private $date = '18-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = array('use_plaxo_abook', 'plaxouser', 'plaxopass');
  static private $config_dist = 'config.inc.php.dist';

  function init()
  {    
    $this->add_texts('localization/', false);
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    
    $rcmail = rcmail::get_instance();
    $this->user = $rcmail->config->get('plaxouser');
    $this->pass = $rcmail->config->get('plaxopass');
    
    if($this->user && $this->pass){
      $this->pass = $rcmail->decrypt($this->pass);
      $this->add_hook('addressbooks_list', array($this, 'addressbooks_list'));
      $this->add_hook('addressbook_get', array($this, 'addressbook_get'));
      $this->add_hook('contact_create', array($this, 'contact_create'));    
      $this->add_hook('contact_update', array($this, 'contact_update'));
      $this->add_hook('contact_delete', array($this, 'contact_delete'));
      $this->add_hook('render_page', array($this, 'render_page'));
      
      // use this address book for autocompletion queries
      $config = $rcmail->config;
      $sources = $config->get('autocomplete_addressbooks', array('sql'));
        
      if (!in_array($this->abook_id, $sources) && $config->get('use_plaxo_abook')){
        $sources[] = $this->abook_id;
        $config->set('autocomplete_addressbooks', $sources);
      }      
    }
    $this->add_hook('preferences_sections_list', array($this, 'addressbooksLink'));
    $this->add_hook('preferences_list', array($this, 'settings_table'));
    $this->add_hook('preferences_save', array($this, 'save_prefs'));
    
    $this->register_action('plugin.plaxo_contacts_uninstall', array($this, 'uninstall'));
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function uninstall(){
    $rcmail = rcmail::get_instance();
    if($rcmail->user->ID){
      $db_table = $rcmail->config->get('plaxo_contacts');
      $query = "DELETE FROM $db_table WHERE user_id=?";
      $rcmail->db->query($query, $rcmail->user->ID);
    }
  }

  function render_page($p){
    if($p['template']== 'addressbook'){
      $rcmail = rcmail::get_instance();
      $script = "rcmail.add_onload(\"rcmail.command('list',rcmail.env.source,false);\");";
      $rcmail->output->add_script($script,'foot');
    }
    return $p;
  }
  
  function contact_create($a){
    if($a['source'] === $this->abook_id){
      //@ ToDo: implement plaxo contacts update when RC supports more fields
      $a['abort'] = true;
    }
    return $a;
  }  
  
  function contact_update($a){
    if($a['source'] === $this->abook_id){
      //@ ToDo: implement plaxo contacts update when RC supports more fields
      $rcmail = rcmail::get_instance(); 
      $rcmail->output->show_message('plaxo_contacts.abookreadonly', 'error', null, false);
      rcmail_overwrite_action('show');
      $a['abort'] = true;
    }
    return $a;
  }
  
  function contact_delete($a){
    if($a['source'] === $this->abook_id){
      //@ ToDo: implement plaxo contacts update when RC supports more fields
      $a['abort'] = true;
    }
    return $a;
  }
    
  function addressbooks_list($p)
  {
    $rcmail = rcmail::get_instance();
    if ($rcmail->config->get('use_plaxo_abook'))
      $p['sources'][$this->abook_id] = 
        array('id' => $this->abook_id, 'name' => Q($this->gettext('plaxocontacts')), 'readonly' => true, 'groups' => false);
    return $p;  
  }
  
  function addressbook_get($p)
  {
    $rcmail = rcmail::get_instance();
    if (($p['id'] === $this->abook_id) && $rcmail->config->get('use_plaxo_abook')) {
        require_once(dirname(__FILE__) . '/plaxo_contacts_backend.php');
        $p['instance'] = new plaxo_contacts_backend($rcmail->db, $rcmail->user->ID);
        $p['instance']->groups = false;
        $this->sync_contacts();
        $rcmail->output->command('enable_command','add','import',false);
    }
    else{
      if ($p['id'] == $rcmail->config->get('default_addressbook',0))
        $rcmail->output->command('enable_command','import',true);
    }
    return $p;
  }
  
  function addressbooksLink($args)
  {
    $temp = $args['list']['server'];
    unset($args['list']['server']);
    $args['list']['addressbook']['id'] = 'addressbook';
    $args['list']['addressbook']['section'] = $this->gettext('addressbook');
    $args['list']['server'] = $temp;

    return $args;
  }
    
  function settings_table($args)
  {
    if ($args['section'] == 'addressbook') {
      $rcmail = rcmail::get_instance();    
      $use_plaxo_abook = $rcmail->config->get('use_plaxo_abook');
      $field_id = 'rcmfd_use_plaxo_abook';
      $checkbox = new html_checkbox(array('name' => '_use_plaxo_abook', 'id' => $field_id, 'value' => 1));
      $args['blocks']['plaxocontacts']['name'] = $this->gettext('plaxocontacts');
      $args['blocks']['plaxocontacts']['options']['use_plaxo_abook'] = array(
        'title' => html::label($field_id, Q($this->gettext('useplaxoabook'))),
        'content' => $checkbox->show($use_plaxo_abook?1:0),
      );
      $field_id = 'rcmfd_plaxo_user';
      $input_plaxouser = new html_inputfield(array('name' => '_plaxouser', 'id' => $field_id, 'size' => 35, 'placeholder' => $this->gettext('username')));
      $args['blocks']['plaxocontacts']['options']['plaxouser'] = array(
        'title' => html::label($field_id, Q($this->gettext('plaxouser'))),
        'content' => $input_plaxouser->show($rcmail->config->get('plaxouser')),
      );
      
      $field_id = 'rcmfd_plaxo_pass';
      if($rcmail->config->get('plaxopass'))
        $title = $this->gettext('plaxopassisset');
      else
        $title = $this->gettext('plaxopassisnotset');
      $input_plaxopass = new html_passwordfield(array('name' => '_plaxopass', 'id' => $field_id, 'size' => 35, 'title' => $title, 'placeholder' => $title));
      $args['blocks']['plaxocontacts']['options']['plaxopass'] = array(
        'title' => html::label($field_id, Q($this->gettext('plaxopass'))),
        'content' => $input_plaxopass->show(),
      );      

    }
    return $args;
  }

  function save_prefs($args)
  {
    if ($args['section'] == 'addressbook') {    
      $rcmail = rcmail::get_instance();
      $args['prefs']['use_plaxo_abook'] = isset($_POST['_use_plaxo_abook']) ? true : false;
      $args['prefs']['plaxouser'] = get_input_value('_plaxouser', RCUBE_INPUT_POST);
      $pass = get_input_value('_plaxopass', RCUBE_INPUT_POST);
      if($pass){
        $args['prefs']['plaxopass'] = $rcmail->encrypt($pass);
      }
    }
    return $args;
  }
    
  function sync_contacts()
  {
    if(isset($_SESSION['plaxo_contacts_sync']))
      return;
      
    $this->get_contacts();   
    if(!is_array($this->results)){
      return;
    }
    
    $rcmail = rcmail::get_instance();    
    require_once(dirname(__FILE__) . '/plaxo_contacts_backend.php');
    $CONTACTS = new plaxo_contacts_backend($rcmail->db, $rcmail->user->ID);
    $this->get_contacts();
    
    $db_table = $rcmail->config->get('db_table_plaxo_contacts','plaxo_contacts');
    $query = "DELETE FROM $db_table WHERE user_id=?";
    $res = $rcmail->db->query($query, $rcmail->user->ID);
    $obj = (array) $this->results;
    
    foreach($obj as $key => $val){
      $insert_id = array();
      $contacts = array();
      $emails = array();
      $pref = "PREF;";
      $vcard  = "BEGIN:VCARD\r\nVERSION:3.0\r\n";
      $vcard .= "N:" . $val->name->familyName . ";" . $val->name->givenName . ";" . $val->name->middleName . "\r\n";
      $vcard .= "FN:" . $val->name->formatted . "\r\n";
      $vcard .= "ORG:" . $val->organizations[0]->name . "\r\n";
      $vcard .= "TITLE:" . $val->organizations[0]->title . "\r\n";      
      $vcard .= "X-AB-EDIT:" . $val->id . "\r\n";
      $vcard .= "BDAY:" . $val->birthday . "\r\n";
      $vcard .= "REV:" . $val->updated . "\r\n";
      if(is_array($val->emails)){           
        foreach($val->emails as $idx => $address){
          if(empty($emails[$address->value])){
            $emails[$address->value] = TRUE;
            $vcard .= "EMAIL;" . $pref . "INTERNET:" . $address->value . "\r\n";
            $pref = "";

            if($val->name->formatted)
              $sname = $val->name->formatted . ' (' . $address->value . ')';
            else
              $sname = $address;
            $contact = array(
              'email' => $address->value,
              'name' => $sname,
              'firstname' => $val->name->givenName,
              'surname' => $val->name->familyName,
              'vcard' => ''
            );
            $contacts[] = $contact;
            $insert_id[] = $CONTACTS->insert($contact, false);
          }
        }
      }
      if(!$contacts[0]){
        $contact = array(
          'email' => '',
          'name' => $val->name->formatted,
          'firstname' => $name[0],
          'surname' => $name[1],
          'vcard' => ''
        );
        $insert_id[] = $CONTACTS->insert($contact, false);
      }
      if(is_array($val->phoneNumbers)){
        foreach($val->phoneNumbers as $idx => $number){
          $vcard .= "TEL;TYPE=" . strtoupper($number->type) . ":" . $number->value . "\r\n";
        }
      }
      if(is_array($val->urls)){
        foreach($val->urls as $idx => $site){
          $vcard .= "URL;TYPE=" . strtoupper($site->type) . ":" . $site->value . "\r\n";
        }
      }      
      if(is_array($val->addresses)){
        foreach($val->addresses as $idx => $address){
          $vcard .= "ADR;TYPE=" . strtoupper($address->type) . ":;;" . str_replace("\r\n",";",$address->formatted) . "\r\n";
        }
      }
      $vcard .= 'END:VCARD';
      $contacts[0]['vcard'] = $vcard;
      $CONTACTS->update($insert_id[0],$contacts[0]); 
    }
    $_SESSION['plaxo_contacts_sync'] = true;
  }
    
  function get_contacts()
  {
    // set credentials for ClientLogin authentication
    $user = $this->user;
    $pass = $this->pass;
    spl_autoload_unregister('rcube_autoload');        
    try {
      $rcmail = rcmail::get_instance();
      if(function_exists('curl_init')){
        $url = "http://www.plaxo.com/pdata/contacts/"; 
        $ch = curl_init();     
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);  
        curl_setopt($ch, CURLOPT_URL, $url);  
        curl_setopt($ch, CURLOPT_USERPWD, $this->user.':' . $this->pass);
        if($rcmail->config->get('http_proxy'))
          curl_setopt($process, CURLOPT_PROXY, $this->proxy . ":" . $rcmail->config->get('http_proxy_port')); 
        $results = json_decode(curl_exec($ch));  
        curl_close($ch);
      } 
      else{      
        require_once "http_request.php";
        $request = new http_request('GET','www._plaxo.com','/pdata/contacts');
        if($rcmail->config->get('http_proxy'))
          $request->proxy($rcmail->config->get('http_proxy'),$rcmail->config->get('http_proxy_port'));
        $request->add_header('User-Agent','MyRoundCube');
        $request->add_header('Authorization', 'Basic ' . base64_encode($this->user.':' . $this->pass));
        $request->send();
        $response = $request->response;
        $temparr = explode("\r\n\r\n",$response);
        $temparr = explode("\n",$temparr[1]);
        $results = json_decode($temparr[1]);
      }
    }
    catch (Exception $e) {
      $this->error = $e->getMessage();  
    }
    spl_autoload_register('rcube_autoload');      
    $this->results = $results->entry;
  }
}
?>
