<?php
/***************************************************
 *
 * CONFIGURATION
 *
 ***************************************************/
 
$dbtype        = 'mysql';
$dbhost        = 'localhost';
$dbname        = 'sabredav';
$dbuser        = 'root';
$dbpass        = 'pass';
$realm         = 'SabreDAV';
$admin         = 'admin@yourdomain.tld';

/***************************************************
 *
 * SabreDAV Server Script
 *
 ***************************************************/
 
$request = explode('/', $_SERVER['REQUEST_URI']);
if($request[1])
  $request = $request[1];
else
  $request = '/';
$pdo = new PDO($dbtype . ':host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
$pdo->setAttribute(PDO::ATTR_ERRMODE,PDO::ERRMODE_EXCEPTION);

//Mapping PHP errors to exceptions 
function exception_error_handler($errno, $errstr, $errfile, $errline ) {
    throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
}
set_error_handler("exception_error_handler");

// Files we need
require_once 'lib/Sabre/autoload.php';

// Backends
$authBackend = new Sabre_DAV_Auth_Backend_PDO($pdo);

switch($request){
  case '/':
  case 'principals':
  case 'calendars':
    $principalBackend = new Sabre_DAVACL_PrincipalBackend_PDO($pdo);
    $calendarBackend = new Sabre_CalDAV_Backend_PDO($pdo);

    // Directory tree
    $tree = array(
            // calendar sharing
            //new Sabre_CALDAV_Principal_Collection($principalBackend),
            // default
            new Sabre_DAVACL_PrincipalCollection($principalBackend),
            new Sabre_CalDAV_CalendarRootNode($principalBackend, $calendarBackend)
    );
    break;
  case 'addressbooks':
    $principalBackend = new Sabre_DAVACL_PrincipalBackend_PDO($pdo);
    $carddavBackend   = new Sabre_CardDAV_Backend_PDO($pdo);
    
    // Directory tree //
    $tree = array(
             new Sabre_DAVACL_PrincipalCollection($principalBackend),
             new Sabre_CardDAV_AddressBookRoot($principalBackend, $carddavBackend),
    );
    break;
  default:
    $tree = null;
}

// The object tree needs in turn to be passed to the server class
$server = new Sabre_DAV_Server($tree);

// You are highly encouraged to set your WebDAV server base url. Without it,
// SabreDAV will guess, but the guess is not always correct. Putting the
// server on the root of the domain will improve compatibility. 
$server->setBaseUri('/');

// Authentication plugin
$authPlugin = new Sabre_DAV_Auth_Plugin($authBackend, $realm);
$server->addPlugin($authPlugin);

// CalDAV plugin
$server->addPlugin(new Sabre_CalDAV_Plugin());

// CardDAV plugin
$server->addPlugin(new Sabre_CardDAV_Plugin());

// ACL plugin
$server->addPlugin(new Sabre_DAVACL_Plugin());

// Support for html frontend
if(isset($_SERVER['PHP_AUTH_DIGEST']) && stripos($_SERVER['PHP_AUTH_DIGEST'], $admin) !== false){
  $browser = new Sabre_DAV_Browser_Plugin();
  $server->addPlugin($browser);
}

// Export Plugin
$icsPlugin = new Sabre_CalDAV_ICSExportPlugin();
$server->addPlugin($icsPlugin);

// And off we go!
$server->exec();

?>