<?php
/**
 * summary
 *
 * @version 3.9.3 - 13.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/
 
class summary extends rcube_plugin
{
  public $task = 'login|mail|settings';
  
  /* unified plugin properties */
  static private $plugin = 'summary';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '3.9.3';
  static private $date = '13-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = array('nosummary');
  static private $config_dist = 'config.inc.php.dist';
  
  function init(){
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
  
    $this->add_texts('localization/', false);
    $this->add_hook('login_after', array($this, 'login_after'));
    $this->add_hook('template_object_summary_mailboxes', array($this, 'summary_html'));
    $this->add_hook('template_object_summary_timezone', array($this, 'summary_html_timezone'));
    $this->add_hook('template_object_summary_disable', array($this, 'summary_html_disable'));
    $this->add_hook('template_object_summary_motd', array($this, 'summary_html_motd'));
    $this->add_hook('template_object_summary_quota', array($this, 'summary_html_quota'));
    $this->add_hook('preferences_list', array($this, 'prefs_table'));
    $this->add_hook('preferences_save', array($this, 'save_prefs'));
    $this->register_action('plugin.summary_getClientTimezone', array($this, 'getClientTimezone'));
    $this->register_action('plugin.summary', array($this, 'summary_startup'));
    $this->register_action('plugin.summary_expunge', array($this, 'summary_expunge'));
    $this->register_action('plugin.summary_purge', array($this, 'summary_purge'));
    $this->register_action('plugin.summary_disable', array($this, 'summary_disable'));    
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function getClientTimezone(){
    $rcmail = rcmail::get_instance();
    $cltz = (int) get_input_value('_cltz', RCUBE_INPUT_POST) * 60;
    $m = '+';
    if($cltz > 0)
      $m = '-';
    $cltz = $m . date('H:i', strtotime('2000-06-30 24:00:00') - $cltz);
    $html = $this->_summary_html_timezone($cltz, get_input_value('_dst', RCUBE_INPUT_POST));
    $rcmail->output->command('plugin.summary_getClientTimezone', $html);
  }
  
  function login_after($args){
    $rcmail = rcmail::get_instance();
    $args['_task'] = 'mail';
    $args['_action'] = 'plugin.summary';
    return $args;
  }
  
  function summary_startup(){
    $rcmail = rcmail::get_instance();
    if(!$rcmail->config->get('nosummary')||
       ($rcmail->config->get('alert_quota_on',true) && $this->_alert_quota_do()== true)){
      $skin  = $rcmail->config->get('skin','classic');
      if(!file_exists("plugins/summary/skins/$skin/summary.css"))
        $skin = "classic";
      $this->include_stylesheet('skins/' . $skin . '/summary.css');
      $rcmail->output->send("summary.summary");
    }
    else{
      $rcmail->output->redirect(array('_action' => '', '_mbox' => 'INBOX'));
    } 
  }
  
  function prefs_table($args){
    if ($args['section'] == 'general') {
      $rcmail = rcmail::get_instance();
      $nosummary= $rcmail->config->get('nosummary');
      $field_id = 'rcmfd_summary';
      $checkbox = new html_checkbox(array('name' => '_nosummary', 'id' => $field_id, 'value' => 1));
      $args['blocks']['main']['options']['summary']['title'] = Q($this->gettext('summary.dontusesummary'));
      $args['blocks']['main']['options']['summary']['content'] = $checkbox->show($rcmail->config->get('nosummary'), array('name' => "_nosummary"));      
    }
    return $args;
  }

  private function _alert_quota_do(){
    $rcmail = rcmail::get_instance();
    $quota = $rcmail->imap->get_quota();
    $quota = $rcmail->plugins->exec_hook('quota', $quota);
    $ret = false;
    if(empty($quota['total'])){
      $ret = false;
    }
    else if($quota['percent'] > $rcmail->config->get('alert_quota_pct',80)){
      $ret = true;
    }
    return $ret;
  }

  function save_prefs($args){
    if($args['section'] == 'general'){
      $args['prefs']['nosummary'] = get_input_value('_nosummary', RCUBE_INPUT_POST);
      return $args;
    }
  }
    
  function summary_disable(){
    if($_POST['_summarydisable'] == 1){
      $rcmail = rcmail::get_instance();
      $a_prefs['nosummary'] = 1;
      $rcmail->user->save_prefs($a_prefs);
      $rcmail->output->redirect(array('_action' => '', '_mbox' => 'INBOX'));
    }
    return;
  }
  
  function summary_expunge(){
    $rcmail = rcmail::get_instance();
    $rcmail->imap->expunge(urldecode($_GET['_mbox']));
    $this->login_after(array());
    return;
  }
  
  function summary_purge(){
    $rcmail = rcmail::get_instance();
    $rcmail->imap->clear_mailbox(urldecode($_GET['_mbox']));
    $this->login_after(array());
    return;  
  }
  
  function summary_html_quota($args){
    $rcmail = rcmail::get_instance();
    $quota = $rcmail->imap->get_quota();
    $quota = $rcmail->plugins->exec_hook('quota', $quota);
    if(empty($quota['total'])){
      $args['content'] = '<div id="summary_quota_container"></div>';
      return $args;
    }
      
    if($quota['percent'] > $rcmail->config->get('alert_quota_pct',80)) 
      $rcmail->output->show_message('summary.quotawarning', 'warning');
    if($quota['percent'] > 99) 
      $rcmail->output->show_message('summary.quotafull', 'error');
    
    $content .= '&nbsp;<table class="quota" ><tr><td align="left" width="' . $quota['percent'] . '%" class="quotafull">&nbsp;' . $quota['percent'] . '%&nbsp;</td><td align="right" class="quotafree" width="' . $quota['free'] . '%">&nbsp;' . $quota['free'] . '%&nbsp;</td></tr></table>';
    
    $quota  = '<fieldset><legend>&nbsp;' . $this->gettext('quota') . '&nbsp;(' . $quota['used'] . ' / ' . $quota['total'] . ' kBytes)</legend>';
    $quota .= $content;
    $quota .= '</fieldset>';
    
    $args['content'] = $quota;
  
    return $args;
  }
  
  function summary_html_motd($args){
    $rcmail = rcmail::get_instance();
    if(file_exists("./plugins/summary/motd/" . $_SESSION['language'] . ".html"))
      $content = file_get_contents("./plugins/summary/motd/" . $_SESSION['language'] . ".html");
    else if(file_exists("./plugins/summary/motd/" . $rcmail->config->get('language', 'en_US') . ".html"))
      $content = file_get_contents("./plugins/summary/motd/" . $rcmail->config->get('language', 'en_US') . ".html");
    else if(file_exists("./plugins/summary/motd/en_US.html"))
      $content = file_get_contents("./plugins/summary/motd/en_US.html");
    else
      $content = file_get_contents("./plugins/summary/motd/en_US.html.dist");
    $motd  = '<fieldset><legend>' . $this->gettext('motd') . '</legend>';
    $motd .= $content;
    $motd .= '</fieldset>';
    $args['content'] = $motd;
    return $args;
  }
  
  function summary_html_timezone($args){
    $args['content'] = '<div id="summary_timezone_container"></div>';
    return $args;
  }
  
  function _summary_html_timezone($soffset,$dst){
    $rcmail = rcmail::get_instance();
    $dst_html = '';
    if($dst == 1){
      $dst_html = '&nbsp;[' . $this->gettext('dst') . ']';
    }
    $tz = $rcmail->config->get('timezone');
    if(is_numeric($tz) || $tz == 'auto'){
      if($rcmail->config->get('dst_active') && is_numeric($tz)){
        $dst_html = '&nbsp;[' . $this->gettext('dst') . ']';
      }
      $tza = $this->get_timezones();
      if($tz == 'auto'){
        $tzas = "&nbsp;" . $tza[$soffset];
      }
      else{
        $tzas = gmdate('H:i', abs($tz) * 3600);
        if($tz >= 0)
          $tzas = '+' . $tzas;
        else
          $tzas = '-' . $tzas;
        $soffset = $tzas;
        $tzas = "&nbsp;" . $tza[$soffset];
      }
    }
    else{
      $stz = date_default_timezone_get();
      date_default_timezone_set($tz);
      $soffset = date('Z',time()) / 3600;
      $tzas = "&nbsp;" . str_replace('_', ' ', $tz);
      if(date('I', time())){
        $dst_html = '&nbsp;[' . $this->gettext('dst') . ']';
      }
      date_default_timezone_set($stz);
    }
    if($soffset >= 0)
      $soffset = '+' . $soffset;
    else
      $soffset = '-' . $soffset;
    $soffset = str_replace('++', '+', $soffset);
    $soffset = str_replace('--', '-', $soffset);
    $html  = '<fieldset><legend>&nbsp;' . $this->gettext('timezone') . '&nbsp;</legend>';
    $html .= '<br /><div>&nbsp;<a href="./?_task=settings">' . $this->gettext('timezone') . '</a>:&nbsp;GMT&nbsp;' . $soffset . $tzas . $dst_html . '</div>';
    $html .= '<br /></fieldset>';
    
    return $html;
  }
  
  function summary_html_disable($args){
    $html  = '<br />';
    $html .= '<form name="f" method="post" action="./?_action=plugin.summary_disable">';
    $html .= '<table width="100%"><tr><td align="right">';
    $html .= $this->gettext('disablesummary') . '&nbsp;' . '<input name="_summarydisable" value="1" onclick="document.forms.f.submit()" type="checkbox" />&nbsp;';
    $html .= '</td></tr></table>';
    $html .= '</form>';
    $args['content'] = $html;
    return $args;
  }  
  
  function summary_html($args){
    $rcmail = rcmail::get_instance();
    $inbox_stat_unseen  = $rcmail->imap->count('INBOX', 'UNSEEN', TRUE);
    $inbox_stat_all     = $rcmail->imap->count('INBOX', 'ALL', TRUE);
    $sent_stat_unseen   = $rcmail->imap->count($rcmail->config->get('sent_mbox'), 'UNSEEN', TRUE);
    $sent_stat_all      = $rcmail->imap->count($rcmail->config->get('sent_mbox'), 'ALL', TRUE);
    $junk_stat_unseen   = $rcmail->imap->count($rcmail->config->get('junk_mbox'), 'UNSEEN', TRUE);
    $junk_stat_all      = $rcmail->imap->count($rcmail->config->get('junk_mbox'), 'ALL', TRUE);
    $trash_stat_unseen  = $rcmail->imap->count($rcmail->config->get('trash_mbox'), 'UNSEEN', TRUE);
    $trash_stat_all     = $rcmail->imap->count($rcmail->config->get('trash_mbox'), 'ALL', TRUE);
    $drafts_stat_unseen  = $rcmail->imap->count($rcmail->config->get('drafts_mbox'), 'UNSEEN', TRUE);
    $drafts_stat_all     = $rcmail->imap->count($rcmail->config->get('drafts_mbox'), 'ALL', TRUE);

    $this->include_script('summary.js');
    
    $user = $rcmail->output->current_username(null);
    if($_SESSION['global_alias'])
      $user = $_SESSION['global_alias'];
    $rcmail->output->add_label('purgefolderconfirm');
    
    $html  = '<fieldset><legend>&nbsp;' . $this->gettext('mailbox') . " ::: " . $user . '&nbsp;</legend><br />';
    $html .= '<ul id="mailboxlist" class="listing">';
    $html .= '<table cellspacing="0" cellpadding="0" id="summarytable">';
    $html .= '<tr>';
    $html .= '<td>';
    $html .= '<li class="mailbox inbox">';
    $html .= '<a id="summaryinbox" class="summaryboxlink" href="./?_task=mail&_mbox=INBOX">'. $this->gettext('inbox') . '&nbsp;&nbsp;&nbsp;&nbsp;</a></td><td align="left">(</td><td align="right"><b>' . $inbox_stat_unseen . '</b></td><td>&nbsp;/&nbsp;</td><td align="right">' . $inbox_stat_all . '</td><td>)';
    $html .= '</li>';
    $html .= '</td><td>&nbsp;&nbsp;&nbsp;&nbsp;[</td>';
    $html .= '<td class="summaryaction summaryok">';
    $html .= '<a class= "summaryok" href="./?task=_mail&_action=plugin.summary_expunge&_mbox=' . urlencode('inbox') . '">' . rcube_label('compact') . '</a>';
    $html .= '</td><td>]</td>';
    $html .= '<td class="summaryaction summarywarning">&nbsp;</td>';
    $html .= '<td width="70%">&nbsp;</td>';
    $html .= '</tr>';
    
    $html .= '<tr>';
    $html .= '<td>';
    $html .= '<li class="mailbox junk">';
    $html .= '<a class="summaryboxlink" href="./?_task=mail&amp;_mbox=' . $rcmail->config->get('junk_mbox') . '">'. $this->gettext('junk') . '&nbsp;&nbsp;&nbsp;&nbsp;</a></td><td align="left">(</td><td align="right"><b>' . $junk_stat_unseen . '</b></td><td>&nbsp;/&nbsp;</td><td align="right">' . $junk_stat_all . '</td><td>)';
    $html .= '</li>';
    $html .= '</td><td>&nbsp;&nbsp;&nbsp;&nbsp;[</td>';
    $html .= '<td class="summaryaction summaryok">';
    $html .= '<a class= "summaryok" href="./?_task=mail&_action=plugin.summary_expunge&_mbox=' . urlencode($rcmail->config->get('junk_mbox')) . '">' . rcube_label('compact') . '</a>';
    $html .= '</td><td>]</td>';
    $warning = "";
    if($junk_stat_all > $rcmail->config->get('junk_warning',true))
      $warning = $this->gettext('cleanup');
    $html .= '<td nowrap class="summaryaction">&nbsp;&nbsp;&nbsp;&nbsp;<a class="summarywarning" href="javascript:emptyfolder(\'' . $rcmail->config->get('junk_mbox') . '\')">' . $warning . '</a>&nbsp;</td>';
    $html .= '<td width="98%">&nbsp;</td>';
    $html .= '</tr>';
        
    $html .= '<tr>';
    $html .= '<td>';
    $html .= '<li class="mailbox sent">';
    $html .= '<a class="summaryboxlink" href="./?_task=mail&amp;_mbox=' . $rcmail->config->get('sent_mbox') . '">'. $this->gettext('sent') . '&nbsp;&nbsp;&nbsp;&nbsp;</a></td><td align="left">(</td><td align="right"><b>' . $sent_stat_unseen . '</b></td><td>&nbsp;/&nbsp;</td><td align="right">' . $sent_stat_all . '</td><td>)';
    $html .= '</li>';
    $html .= '</td><td>&nbsp;&nbsp;&nbsp;&nbsp;[</td>';
    $html .= '<td class="summaryaction summaryok">';
    $html .= '<a class= "summaryok" href="./?_task=mail&_action=plugin.summary_expunge&_mbox=' . urlencode($rcmail->config->get('sent_mbox')) . '">' . rcube_label('compact') . '</a>';
    $html .= '</td><td>]</td>';
    $warning = "";
    if($sent_stat_all > $rcmail->config->get('sent_warning',true))
      $warning = $this->gettext('cleanup');
    $html .= '<td nowrap class="summaryaction">&nbsp;&nbsp;&nbsp;&nbsp;<a class="summarywarning" href="javascript:emptyfolder(\'' . $rcmail->config->get('sent_mbox') . '\')">' . $warning . '</a>&nbsp;</td>';
    $html .= '<td width="98%">&nbsp;</td>';
    $html .= '</tr>';
    
    $html .= '<tr>';
    $html .= '<td>';
    $html .= '<li class="mailbox trash">';
    $html .= '<a class="summaryboxlink" href="./?_task=mail&amp;_mbox=' . $rcmail->config->get('trash_mbox') . '">'.$this->gettext('trash') . '&nbsp;&nbsp;&nbsp;&nbsp;</a></td><td align="left">(</td><td align="right"><b>' . $trash_stat_unseen . '</b></td><td>&nbsp;/&nbsp;</td><td align="right">' . $trash_stat_all . '</td><td>)';
    $html .= '</li>';
    $html .= '</td><td>&nbsp;&nbsp;&nbsp;&nbsp;[</td>';
    $html .= '<td class="summaryaction summaryok">';
    $html .= '<a class= "summaryok" href="./?_task=mail&_action=plugin.summary_expunge&_mbox=' . urlencode($rcmail->config->get('trash_mbox')) . '">' . rcube_label('compact') . '</a>';
    $html .= '</td><td>]</td>';
    $warning = "";
    if($trash_stat_all > $rcmail->config->get('trash_warning',true))
      $warning = $this->gettext('cleanup');
    $html .= '<td nowrap class="summaryaction">&nbsp;&nbsp;&nbsp;&nbsp;<a class="summarywarning" href="javascript:emptyfolder(\'' . $rcmail->config->get('trash_mbox') . '\')">' . $warning . '</a>&nbsp;</td>';
    $html .= '<td width="98%">&nbsp;</td>';
    $html .= '</tr>';    

    $html .= '<tr>';
    $html .= '<td>';
    $html .= '<li class="mailbox drafts">';
    $html .= '<a class="summaryboxlink" href="./?_task=mail&amp;_mbox=' . $rcmail->config->get('drafts_mbox') . '">'. $this->gettext('drafts') . '&nbsp;&nbsp;&nbsp;&nbsp;</a></td><td align="left">(</td><td align="right"><b>' . $drafts_stat_unseen . '</b></td><td>&nbsp;/&nbsp;</td><td align="right">' . $drafts_stat_all . '</td><td>)';
    $html .= '</li>';
    $html .= '</td><td>&nbsp;&nbsp;&nbsp;&nbsp;[</td>';
    $html .= '<td class="summaryaction summaryok">';
    $html .= '<a class= "summaryok" href="./?_task=mail&_action=plugin.summary_expunge&_mbox=' . urlencode($rcmail->config->get('drafts_mbox')) . '">' . rcube_label('compact') . '</a>';
    $html .= '</td><td>]</td>';
    $warning = "";
    if($drafts_stat_all > 0)
      $warning = $this->gettext('unfinished');
    $html .= '<td nowrap class="summaryaction">&nbsp;&nbsp;&nbsp;&nbsp;<a class="summarywarning" href="javascript:gotofolder(\'' . $rcmail->config->get('drafts_mbox') . '\')">' . $warning . '</a>&nbsp;</td>';
    $html .= '<td width="98%">&nbsp;</td>';
    $html .= '</tr>';

    $html .= '</table>';
    
    $html .= '</ul>';
    $html .= '<br /></fieldset>';
    
    $args['content'] = $html;

    return $args;
  }
  
  function get_timezones()
  {
    $tza = array();
    $tza['-11:00'] = 'Midway Island, Samoa';
    $tza['-10:00'] = 'Hawaii';
    $tza['-09:30'] = 'Marquesas Islands';
    $tza['-09:00'] = 'Alaska';
    $tza['-08:00'] = 'Pacific Time (US/Canada)';
    $tza['-07:00'] = 'Mountain Time (US/Canada)';
    $tza['-06:00'] = 'Central Time (US/Canada), Mexico City';
    $tza['-05:00'] = 'Eastern Time (US/Canada), Bogota, Lima';
    $tza['-04:30'] = 'Caracas';
    $tza['-04:00'] = 'Atlantic Time (Canada), La Paz';
    $tza['-03:30'] = 'Nfld Time (Canada), Nfld, S. Labador';
    $tza['-03:00'] = 'Brazil, Buenos Aires, Georgetown';
    $tza['-02:00'] = 'Mid-Atlantic';
    $tza['-01:00'] = 'Azores, Cape Verde Islands';
    $tza['+00:00'] = '(GMT) Western Europe, London, Lisbon, Casablanca';
    $tza['+01:00'] = 'Central European Time';
    $tza['+02:00'] = 'EET: Tallinn, Helsinki, Kaliningrad, South Africa';
    $tza['+03:00'] = 'Baghdad, Kuwait, Riyadh, Moscow, Nairobi';
    $tza['+03:30'] = 'Tehran';
    $tza['+04:00'] = 'Abu Dhabi, Muscat, Baku, Tbilisi';
    $tza['+04:30'] = 'Kabul';
    $tza['+05:00'] = 'Ekaterinburg, Islamabad, Karachi';
    $tza['+05:30'] = 'Chennai, Kolkata, Mumbai, New Delhi';
    $tza['+05:45'] = 'Kathmandu';
    $tza['+06:00'] = 'Almaty, Dhaka, Colombo';
    $tza['+06:30'] = 'Cocos Islands, Myanmar';
    $tza['+07:00'] = 'Bangkok, Hanoi, Jakarta';
    $tza['+08:00'] = 'Beijing, Perth, Singapore, Taipei';
    $tza['+08:45'] = 'Caiguna, Eucla, Border Village';
    $tza['+09:00'] = 'Tokyo, Seoul, Yakutsk';
    $tza['+09:30'] = 'Adelaide, Darwin';
    $tza['+10:00'] = 'EAST/AEST: Sydney, Guam, Vladivostok';
    $tza['+10:30'] = 'New South Wales';
    $tza['+11:00'] = 'Magadan, Solomon Islands';
    $tza['+11:30'] = 'Norfolk Island';
    $tza['+12:00'] = 'Auckland, Wellington, Kamchatka';
    $tza['+12:45'] = 'Chatham Islands';
    $tza['+13:00'] = 'Tonga, Pheonix Islands';
    $tza['+14:00'] = 'Kiribati';  
    return $tza;
  }  
}

?>