<?php

/**
 * Add a customizeable taskbar to certain pages
 *
 * @version 3.5 - 14.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/
 
class taskbar extends rcube_plugin
{
  public $task = 'login|logout';
  public $noajax = true;
  public $noframe = true;
  
  /* unified plugin properties */
  static private $plugin = 'taskbar';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Please re-configure the plugin. "default" has to become "classic" since Roundcube 0.8.x.';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '3.5';
  static private $date = '14-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_texts('localization/');  
    $this->load_config('cachebuster.inc.php');
    $this->add_hook('render_page', array($this, 'html'));
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  
  function html($p)
  {
    $rcmail = rcmail::get_instance();  
    $templates = $rcmail->config->get('taskbar_templates');
    if(is_array($templates))
      $templates = array_flip($templates);
    else
      $templates = array('login' => 0);
      
    if(isset($templates[$p['template']])){
      $this->include_stylesheet('taskbar.css.php?skin=' . urlencode($rcmail->config->get('skin')) . "&v=" . $rcmail->config->get('fronttasks_version'));

      $def = $rcmail->config->get('login_taskbar_def', array());
      $def = (array) $def[urlencode($rcmail->config->get('skin'))];
      $links = '';
    
      foreach($def as $key => $val){
        if(!isset($def[$key]['plugin']) || class_exists($def[$key]['plugin'])){

          // get localized strings and add labels to client to make them accessable by javascript
          // check first if there is a label in the plugin localization, if not use default labels
          // if there is still no label use definition key

          $innerHTML_locale = $this->gettext($def[$key]['plugin'].".".$def[$key]['innerHTML_locale']);
          if(substr($innerHTML_locale,0,1) == "["){
            $innerHTML_locale =  rcube_label(strtolower($def[$key]['innerHTML_locale']));
          }
          if(substr($innerHTML_locale,0,1) == "["){
            $innerHTML_locale =  $this->gettext($def[$key]['innerHTML_locale']);
          }      
          $innerHTML_locale = str_replace("]","",str_replace("[","",$innerHTML_locale));
          $title_locale = $this->gettext($def[$key]['plugin'].".".$def[$key]['title_locale']);
          if(substr($title_locale,0,1) == "["){
            $title_locale =  rcube_label(strtolower($def[$key]['title_locale']));
          }
          if(substr($title_locale,0,1) == "["){
            $title_locale =  $this->gettext($def[$key]['title_locale']);
          }
          $title_locale = str_replace("]","",str_replace("[","",$title_locale));
      
          // add the link to the output
          if($val['plugin'] == 'pwtools' && $p['template'] != 'login'){
            $links = '<a href="./' .
                     '" target="_self' .
                     '" class="'   . $def[$key]['class'] . 
                     '" title="'   . $this->gettext('login') . 
                     '">' . $this->gettext('login') . "</a>\n" . $links;
          }
          else{
            // show Demo link only on login page
            if($val['plugin'] == 'demologin' && $p['template'] != 'login')
              continue;
            $links .= '<a href="'    . $def[$key]['href'] .
                      '" target="'  . $def[$key]['target'] .
                      '" onclick="' . $def[$key]['onclick'] .
                      '" class="'   . $def[$key]['class'] . 
                      '" title="'   . $title_locale . 
                      '">' . sprintf($def[$key]['innerHTML'], $innerHTML_locale) . "</a>\n";
          }
        }
      }
      if($rcmail->config->get('skin') == 'larry'){
        $logo = $rcmail->config->get('skin_logo');
        if($logo && $p['template'] == 'login'){
          $logo = '<img src="' . $logo . '" id="toplogo" alt="Logo" border="0" />';
        }
        else{
          $logo = '';
        }
        $content = '<div id="header" style="position:absolute; top:0; left:0; right: 0;"><div id="topline"></div><div id="topnav">' . $logo . '<div id="taskbar" class="topright" style="margin-top: 20px;">' . "\n" . $links . "</div></div></div>\n";
      }
      else{
        $content = '<div id="taskbar"><div id="taskbar-contents">' . "\n" . $links . "</div><div id='taskbar-line'></div></div>\n";
      }
    
      $rcmail->output->add_footer($content);
    }
    
    return $p;
    
  }

}

?>